#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Installation script for MrBUMP
# Ronan Keegan 14/12/05

import os, sys, string
import shutil
import time

sys.path.append(os.getcwd() + '/setup_lib')

import Scan_path
import Setup_config
import Gui_guide

# Check for CCP4
if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'

# Check python version
if sys.version_info[0] < 2  or (sys.version_info[0] >= 2 and sys.version_info[1] < 3):
   raise RuntimeError, 'Python version is too old. Python 2.3 or later is required for MrBUMP' 


class Installer:

   def __init__(self):
      self.cur_dir=os.getcwd()
      self.CCP4=os.environ["CCP4"]
      self.chk_dep=Scan_path.Check_PATH()
      self.mrbump_dir=''

      self.anisoanl_found=False
      self.mafft_found=False
      self.clustalw_found=False
      self.gnuplot_found=False
      self.fasta34_found=False

   def check_dependencies(self):
      """ Test the system to see if the dependencies exist. """
     
      print ""
      print "Installing MrBUMP on your system."
      print "Checking for dependencies..."


      # Check version of CCP4

      # Check first for anisonal
      self.anisoanl_found=self.chk_dep.find_exec('anisoanl')
      if self.anisoanl_found==False:
         print ""
         print "Can't find anisoanl in system path"
         print "Can't check CCP4 version"
         print "Please make sure you have CCP4 5.99 or later installed"
         print ""
         time.sleep(2)
      else:
         os.system("anisoanl -i > /tmp/ccp4_ver.txt")

         f=open("/tmp/ccp4_ver.txt")
         r=string.split(string.strip(f.readlines()[0]))[-1]
         f.close()
         os.remove("/tmp/ccp4_ver.txt")
      
         ver1=int(string.split(r,'.')[0])
         ver2=int(string.split(r,'.')[1])
       
         if (ver1 == 5 and ver2 < 99) or (ver1 < 5):
            print ""
            print "MRBump requires CCP4 version 5.99 or later"
            print "CCP4 5.99 can be downloaded from:"
            print "http://www.ccp4.ac.uk/dev/releases-6.0/ccp4-download/download_beta/public/downloadman.php"
            print ""
            sys.exit()
         else:
            print ""
            print "CCP4 version %d.%d detected" % (ver1, ver2)
            print ""

      # Mafft/Clustalw
      self.mafft_found=self.chk_dep.find_exec('mafft')
      self.clustalw_found=self.chk_dep.find_exec('clustalw')

      if self.mafft_found==False and self.clustalw_found==False:
         print ""
         print "Install error: Neither Mafft nor Clustalw were found on your system."
         print "One of these is required to perform the multiple alignment of the"
         print "target and template sequences."
         print ""
         print "Mafft can be downloaded from:"
         print "http://www.biophys.kyoto-u.ac.jp/~katoh/programs/align/mafft/"
         print ""
         print "Clustalw can be downloaded from:"
         print "ftp://ftp.ebi.ac.uk/pub/software/unix/clustalw/"
         print ""
         sys.exit()
      elif self.mafft_found==True:
         print "Mafft multiple alignment program detected."
      else:
         print "Clustalw multiple alignment program detected"
 
      # Gnuplot
      self.gnuplot=self.chk_dep.find_exec('gnuplot')

      if self.gnuplot==False:
         print ""
         print "Install error: Gnuplot was not detected on your system. Gnuplot is"
         print "a required to generate plots of the results from MrBUMP."
         print ""
         print "Gnuplot can be downloaded from:"
         print "http://www.gnuplot.info/"
         print ""
         sys.exit()
      else:
         print "Gnuplot program detected."

      # Fasta34
      self.fasta34=self.chk_dep.find_exec('fasta34')
 
      if self.fasta34==False:
         print ""
         print "Install warning: Fasta34 not found. Fasta34 is an optional dependency"
         print "which can be used to do the sequence-based search on the local machine."
         print "The alternative is to send the target sequence to the OCA web-based"
         print "application which will carry out the sequence-based search for homologous"
         print "structures. If you are senstive about your sequence data you should install"
         print "fasta34 on your machine."
         print ""
         print "Fasta34 can be downloaded from:"
         print "ftp://ftp.ebi.ac.uk/pub/software/unix/fasta/"
         print ""
         time.sleep(2)
      else:
         print "Fasta34 program detected."

      print ""

   def install(self):

      # Copy the tar-ball to the CCP4 folder if the CCP4 folder is writeable by the user.
      # Otherwise unpack the tar-ball in the current folder.
      if os.name=='nt':
         print "Windows not supported with this version of setup.py"
      else:
         if os.access(self.CCP4, os.W_OK):
            shutil.copy("mrbump.tar.gz", self.CCP4)
            os.chdir(self.CCP4)
         else:
            self.mrbump_dir=os.getcwd()
           
      if os.name=='nt':
         print "Windows not supported with this version of setup.py"
         sys.exit()
      else:
         print "Unpacking MrBUMP...."
         os.system("gzip -dc mrbump.tar.gz | tar xf -")
           
      print "Cleaning-up...."

      if os.name=='nt':
         print "Windows not supported with this version of setup.py"
      else:
         if os.access(self.CCP4, os.W_OK):
            os.remove("mrbump.tar.gz")

            shutil.copy(self.CCP4 + '/bin/mrbump.ccp4', self.CCP4 + '/bin/mrbump')
            os.remove(self.CCP4 + '/bin/mrbump.ccp4') 
            os.remove(self.CCP4 + '/bin/mrbump.local') 

            os.chdir(self.cur_dir)
         else:
            shutil.copy(self.mrbump_dir + '/bin/mrbump.local', self.mrbump_dir + '/bin/mrbump')
            os.remove(self.mrbump_dir + '/bin/mrbump.ccp4') 
            os.remove(self.mrbump_dir + '/bin/mrbump.local') 

      print "Installation complete."

      if os.access(self.CCP4, os.W_OK)==False:

         set_config=Setup_config.Make_config()
         set_config.write_config(self.mrbump_dir)

         print ""
         print "#########################################################################"
         print "             IMPORTANT ---- Environment Setup ---- IMPORTANT"
         print "#########################################################################"
         print ""
         print "Before you can run MrBUMP you need to source the setup script:"
         print ""
         print self.mrbump_dir + '/configure/mrbump.setup'
         print ""
         print "We suggest that you source it from your .bashrc or .cshrc file depending on"
         print "the shell you are using."
         print ""
    
      ccp4_gui=Gui_guide.CCP4_GUI()
      if os.access(self.CCP4, os.W_OK):
         ccp4_gui.display_instructions(self.CCP4)
      else:
         ccp4_gui.display_instructions(self.mrbump_dir)

if __name__ == '__main__':
   
   install=Installer()
   install.check_dependencies()
   install.install()

