#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#

import os,sys,string


class Mtzdump:
  '''This is a wrapper for the program mtzdump which will print the
  header of an MTZ file. Methods are provided for extracting certain
  quantities from the header'''

  def __init__(self):
    self.Executable=os.environ["CBIN"] + "/mtzdump"
    self.programParameters = 'END\n'
    self.logfile = ''
    try:
       self.debug=eval(os.environ['MRBUMP_DEBUG'])
    except:
       self.debug=False
  
  def setDEBUG(self, flag):
     self.debug=flag  

  def setMTZdumpLogfile(self, mrsearchdir):
    self.logfile = mrsearchdir + '/logs/junk_mtzdump.log'
   
  def setProgramParameters(self,inputline):
    self.programParameters = inputline + '\n' + self.programParameters

  def setHklin(self, hklin):
    self.hklin = hklin

  def go(self):
    i,o=os.popen2(self.Executable + ' HKLIN ' + self.hklin + ' > ' + self.logfile)
    i.write(self.programParameters)
    i.close()

    o.close()

  def getCell(self):
    f = open(self.logfile,'r')
    logline = f.readline()
    # with "while logline" this loop terminates early
    # don't know why!
    while 1:
      if 'Cell Dimensions' in logline:
        logline = f.readline()
        logline = f.readline()
        cell = string.split(logline)
        f.close()
        return cell
      logline = f.readline()
    f.close()

  def getSymmetryNumber(self):
    f = open(self.logfile,'r')
    logline = f.readline()
    # with "while logline" this loop terminates early
    # don't know why!
    while 1:
      if 'Space group' in logline:
        l = string.split(logline)
        symmetryNumber = l[-1].rstrip(')')
        f.close()
        return symmetryNumber
      logline = f.readline()
    f.close()

  def getResolution(self):
    f = open(self.logfile,'r')
    logline = f.readline()
    # with "while logline" this loop terminates early
    # don't know why!
    while 1:
      if 'Resolution Range' in logline:
        logline = f.readline()
        logline = f.readline()
        resline = string.split(logline)
        resolution = float(resline[-3].rstrip(')'))
        f.close()
        return resolution
      logline = f.readline()
    f.close()

  def getColumnData(self):
    ''' A function to get the column labels and types from an MTZ file. Takes in a dictionary
    to store the labels and the their corresponding column types'''

    # Open the log file for reading

    f = open(self.logfile,'r')
    logline = f.readline()

    # Loop over the lines in the logfile and save the column data information

    while logline:
      if 'Column Labels' in logline:
        logline = f.readline()
        logline = f.readline()
        col_labels = string.split(string.strip(logline))
      #if 'Column Types' in logline:
      #  logline = f.readline()
      #  logline = f.readline()
      #  ctypes = string.split(string.strip(logline))
        break
      logline = f.readline()
  
    # Fill the dictionary with the column details

    #for i in range(len(clabels)):
       #col_data[clabels[i]]=ctypes[i]

    f.close()
    return col_labels



