#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
# Code to handle the keyword input to MRBUMP
# Ronan Keegan 12.10.05

import string
import sys
import os

class Keywords:
   """ A class to read the keywords for MRBUMP from the stdin or input from a file. """

   def __init__(self):
      self.col_labels=dict([])
      self.job_ID=''

      self.ROOTDIR = os.getcwd() 
      self.results_html =  ''
   
      self.NMASU=None
      self.PACK=5
      self.NCYC=30

      self.MRNUM = 10
      self.ENSEMNUM = 5
 
      self.E_value = 0.02

      self.MRPROGRAM = 'MOLREP'
      self.MAPROGRAM = 'MAFFT'
      
      self.SSM = False
      self.SCOP = True 
      self.PQS = True 

      self.MDLDPDBCLP = True 
      self.MDLPLYALA = True 
      self.MDLMOLREP = True 
      self.MDLCHAINSAW = True 

      self.DEBUG = False
      self.CLUSTER = False
      self.fasta_local = False
      self.run_local   = False
      self.ONLYMODELS = False
      self.UPDATE = True
      self.ISOLATED = False

      self.ignore_list = []
      self.include_list = []
      self.error_string = ''

      self.labin_found = False
      self.jobid_found = False

   def setErrorString(self, error_string):
      self.error_string = error_string

   def read_keywords(self):
      """ A function to read in the keyword arguments from the STDIN or from a file. """

      print "ENTER KEYWORD INPUT FROM FILE OR FROM STANDARD INPUT"

      while True:
         data_line = string.strip(raw_input("-->\n"))
         if data_line == '':
           keyword = "NULL"
         else:
           keyword = string.split(data_line)[0]

         if "LABI" in keyword.upper():
            self.parse_LABIN(data_line)
            self.labin_found = True
    
         elif "JOBI" in keyword.upper():
            self.parse_JOBID(data_line)
            self.jobid_found = True
            
         elif "ROOT" in keyword.upper():
            self.parse_ROOTDIR(data_line)

         elif "RESH" in keyword.upper():
            self.parse_RESHTML(data_line)

         elif "IGNO" in keyword.upper():
            self.parse_IGNORE(data_line)

         elif "INCL" in keyword.upper():
            self.parse_INCLUDE(data_line)

         elif "NMAS" in keyword.upper():
            self.parse_NMASU(data_line)

         elif "PACK" in keyword.upper():
            self.parse_PACK(data_line)

         elif "NCYC" in keyword.upper():
            self.parse_NCYC(data_line)

         elif "MRNU" in keyword.upper():
            self.parse_MRNUM(data_line)

         elif "ENSE" in keyword.upper():
            self.parse_ENSEMNUM(data_line)

         elif "EVAL" in keyword.upper():
            self.parse_EVALUE(data_line)

         elif "MRPR" in keyword.upper():
            self.parse_MRPROGRAM(data_line)

         elif "MAPR" in keyword.upper():
            self.parse_MAPROGRAM(data_line)

         elif "MDLD" in keyword.upper():
            self.parse_FLAG(data_line, 'MDLDPDBCLP')

         elif "MDLP" in keyword.upper():
            self.parse_FLAG(data_line, 'MDLPLYALA')

         elif "MDLM" in keyword.upper():
            self.parse_FLAG(data_line, 'MDLMOLREP')

         elif "MDLC" in keyword.upper():
            self.parse_FLAG(data_line, 'MDLCHAINSAW')

         elif "DEBU" in keyword.upper():
            self.parse_FLAG(data_line, 'DEBUG')

         elif "CLUS" in keyword.upper():
            self.parse_FLAG(data_line, 'CLUSTER')

         elif "SSMS" in keyword.upper():
            self.parse_FLAG(data_line, 'SSMSEARCH')

         elif "SCOP" in keyword.upper():
            self.parse_FLAG(data_line, 'SCOPSEARCH')

         elif "PQSS" in keyword.upper():
            self.parse_FLAG(data_line, 'PQSSEARCH')

         elif "PWAL" in keyword.upper():
            self.parse_FLAG(data_line, 'PWALIGN')

         elif "FAST" in keyword.upper():
            self.parse_FLAG(data_line, 'FASTALOCAL')

         elif "RUNL" in keyword.upper():
            self.parse_FLAG(data_line, 'RUNLOCAL')
      
         elif "ONLY" in keyword.upper():
            self.parse_FLAG(data_line, 'ONLYMODELS')
      
         elif "UPDA" in keyword.upper():
            self.parse_FLAG(data_line, 'UPDATE')
      
         elif "ISOL" in keyword.upper():
            self.parse_FLAG(data_line, 'ISOLATED')
      
         elif "NULL" in keyword.upper():
            print "Please enter a keyword or type END to finish keyword entry"
      
         elif "END" in keyword.upper():
            break
       
         else:
            print "Ignoring unrecognised keyword: %s" % keyword 
            
  
      # Verify the 'NUM_' keywords are correct
      self.verify_NUM()

      # Check that the LABIN and JOBID keywords have been provided. All others are optional.
      if self.labin_found == False:
         self.keyword_usage('labin', 'LABIN keyword not provided', 11)

      if self.jobid_found == False:
         self.keyword_usage('jobid', 'JOBID keyword not provided', 12)

   def parse_JOBID(self, data_line):
      """ A function to parse the JOBID keyword. """
      
      if len(string.split(data_line)) == 1:
         print "Keyword entry error: JOBID requires an argument"
      else:
         self.job_ID=string.split(data_line)[1]
      
         print "Job identifier: %s" % self.job_ID

   def parse_ROOTDIR(self, data_line):
      """ A function to parse the ROOT keyword. """
   
      if len(string.split(data_line)) == 1:
         print "Keyword entry error: ROOTDIR requires an argument"
      else:
         self.ROOTDIR=string.split(data_line)[1]
   
         #Test directory existence
         if os.path.isdir(self.ROOTDIR) == False:
            self.keyword_usage('root_dir', '%s does not exist' % self.ROOTDIR, 15)
      
         print "Root directory: %s" % self.ROOTDIR

   def parse_RESHTML(self, data_line):
      """ A function to parse the RESHTML keyword. """
   
      if len(string.split(data_line)) == 1:
         print "Keyword entry error: RESHTML requires an argument"
      else:
         self.results_html=string.split(data_line)[1]
   
         print "Results HTML location: %s" % self.results_html

   def parse_NMASU(self, data_line):
      """ A function to parse the NMASU keyword. """

      if len(string.split(data_line)) == 1:
         print "No value given for NMASU. Number of molecules will be calculated automatically"
      else:
         value = string.split(data_line)[1]

         if value.isdigit():
            self.NMASU=int(value)
         else:
            self.keyword_usage('NMASU', "'%s' is not a valid value for NMASU, must be integer" % value, 4)

         print "Number of molecules in the asu: %d" % self.NMASU

   def parse_PACK(self, data_line):
      """ A function to parse the PACK keyword. """

      if len(string.split(data_line)) == 1:
         print "Keyword entry error: PACK requires an argument"
      else:
         value = string.split(data_line)[1]

         if value.isdigit():
            self.PACK=int(value)
         else:
            self.keyword_usage('PACK', "'%s' is not a valid value for PACK, must be integer" % value, 4)

         print "Number of clashes tolerated in Phaser: %d" % self.PACK

   def parse_NCYC(self, data_line):
      """ A function to parse the NCYC keyword. """

      if len(string.split(data_line)) == 1:
         print "Keyword entry error: NCYC requires an argument"
      else:
         value = string.split(data_line)[1]

         if value.isdigit():
            self.NCYC=int(value)
         else:
            self.keyword_usage('NCYC', "'%s' is not a valid value for NCYC, must be integer" % value, 4)

         print "Number of cycles of restrained refinement to use in Refmac: %d" % self.NCYC

   def parse_MRNUM(self, data_line):
      """ A function to parse the MRNUM keyword. """
   
      if len(string.split(data_line)) == 1:
         print "Keyword entry error: MRNUM requires an argument"
      else:
         value = string.split(data_line)[1]

         if value.isdigit():
            self.MRNUM=int(value)
         else:
            self.keyword_usage('MRNUM', "'%s' is not a valid value for MRNUM, must be integer" % value, 5)
      
         print "Number of search models to use in MR: %d" % self.MRNUM

   def parse_ENSEMNUM(self, data_line):
      """ A function to parse the ENSEMNUM keyword. """
   
      if len(string.split(data_line)) == 1:
         print "Keyword entry error: ENSEMNUM requires an argument"
      else:
         value = string.split(data_line)[1]

         if value.isdigit():
            self.ENSEMNUM=int(value)
         else:
            self.keyword_usage('ENSEMNUM', "'%s' is not a valid value for ENSEMNUM, must be integer" % value, 6)
      
         print "Number of models to use in Phaser ensemble: %d" % self.ENSEMNUM

   def parse_EVALUE(self, data_line):
      """ A function to parse the E_value keyword. """

      if len(string.split(data_line)) == 1:
         print "Keyword entry error: EVALUE requires an argument"
      else:
         value=string.split(data_line)[1]  
 
         try:
            self.E_value=float(value)
         except:
            self.keyword_usage('E_value', "'%s' is not a valid value for EVALUE, must be float" % value, 7)
      
         print "E value to use in fasta search: %.6f" % self.E_value

   def parse_MRPROGRAM(self, data_line):
      """ A function to parse the MRPROGRAM keyword. """

      if len(string.split(data_line)) == 1:
         print "Keyword entry error: MRPROGRAM requires an argument"
      else:
         program = (string.split(data_line)[1]).upper()

         if program != 'MOLREP' and program != 'PHASER' and program != 'AMORE':     
            self.keyword_usage('MRPROGRAM', "'%s' is not a valid MR program" % program, 10)
         else:
            self.MRPROGRAM=program
      
         print "Default MR program: %s" % self.MRPROGRAM

   def parse_MAPROGRAM(self, data_line):
      """ A function to parse the MAPROGRAM keyword. """

      if len(string.split(data_line)) == 1:
         print "Keyword entry error: MAPROGRAM requires an argument"
      else:
         program = (string.split(data_line)[1]).upper()

         if program != 'MAFFT' and program != 'CLUSTALW':     
            self.keyword_usage('MAPROGRAM', "'%s' is not a valid Sequence Multiple Alignment program" % program, 14)
         else:
            self.MAPROGRAM=program
      
         print "Default sequence alignment program: %s" % self.MAPROGRAM

   def parse_IGNORE(self, data_line):
      """ A function to parse the IGNORE keyword. """
   
      # Make a list from the line and delete the keyword IGNORE from the start
      line=string.split(data_line)
      del line[0]
      
      # Verify each PDB ID and add it to the ignore list 
      for i in line:
         pdb_id=i.lower()
         if len(pdb_id) !=4 or pdb_id.isalnum() == False or pdb_id[0].isdigit() == False:
            self.keyword_usage('ignore', "%s is not a valid PDB ID" % pdb_id, 3)
         else:
            self.ignore_list.append(pdb_id)
      print "List of PDB codes to ignore in the search: ", self.ignore_list
      
   def parse_INCLUDE(self, data_line):
      """ A function to parse the INCLUDE keyword. """
   
      # Make a list from the line and delete the keyword INCLUDE from the start
      line=string.split(data_line)
      del line[0]
      
      # Verify each PDB ID and add it to the include list 
      for i in line:
         chain=i.lower()
         if len(chain) !=6 or chain[0].isdigit() == False or chain[4] != "_" or chain[5].isalpha() == False:
            self.keyword_usage('include', "%s is not a valid Chain ID" % chain, 3)
         else:
            self.include_list.append(chain)
      print "List of PDB codes to include in the search: ", self.include_list
      
   def parse_FLAG(self, data_line, env_flag):
      """ A function to parse the any boolen flag keywords. """
  
      flag=string.split(data_line)[1]
       
      # Verify that the variable is set to True or False
      if flag.upper() != 'FALSE' and flag.upper() != 'TRUE' and flag != '1' and flag != '0':
         self.keyword_usage(env_flag, "'%s' is not a valid option for keyword %s" % (flag, env_flag), 15)

      if flag.upper() == 'TRUE' or flag == '1':
         if env_flag == 'DEBUG': 
            self.DEBUG=True
            os.environ["MRBUMP_DEBUG"]="True"
            print "Debug set to: ", self.DEBUG
         if env_flag == 'CLUSTER': 
            self.CLUSTER=True
            os.environ["MRBUMP_CLUSTER"]="True"
            print "Cluster set to: ", self.CLUSTER
         if env_flag == 'SCOPSEARCH':
            self.SCOP=True
            print "SCOP search set to: ", self.SCOP
         if env_flag == 'SSMSEARCH':
            self.SSM=True
            print "SSM search set to: ", self.SSM
         if env_flag == 'PQSSEARCH':
            self.PQS=True
            print "PQS search set to: ", self.PQS
         if env_flag == 'FASTALOCAL':
            self.fasta_local=True
            print "Run local Fasta search set to: ", self.fasta_local
         if env_flag == 'RUNLOCAL':
            self.run_local=True
            print "Run local set to: ", self.run_local
         if env_flag == 'ONLYMODELS':
            self.ONLYMODELS=True
            print "Only generate models set to: ", self.ONLYMODELS
         if env_flag == 'UPDATE':
            self.UPDATE=True
            print "Update database files set to: ", self.UPDATE
         if env_flag == 'ISOLATED':
            self.ISOLATED=True
            print "Run in ISOLATED mode set to: ", self.ISOLATED
         if env_flag == 'MDLDPDBCLP':
            self.MDLDPDBCLP=True
            print "Generate PDBClip models set to: ", self.MDLDPDBCLP
         if env_flag == 'MDLPLYALA':
            self.MDLPLYALA=True
            print "Generate Polyalanine models set to: ", self.MDLPLYALA
         if env_flag == 'MDLMOLREP':
            self.MDLMOLREP=True
            print "Generate Molrep models set to: ", self.MDLMOLREP
         if env_flag == 'MDLCHAINSAW':
            self.MDLCHAINSAW=True
            print "Generate Chainsaw models set to: ", self.MDLCHAINSAW

      if flag.upper() == 'FALSE' or flag == '0': 
         if env_flag == 'DEBUG': 
            self.DEBUG=False
            os.environ["MRBUMP_DEBUG"]="False"
            print "Debug set to: ", self.DEBUG
         if env_flag == 'CLUSTER': 
            self.CLUSTER=False
            os.environ["MRBUMP_CLUSTER"]="False"
            print "Cluster set to: ", self.CLUSTER
         if env_flag == 'SCOPSEARCH':
            self.SCOP=False
            print "SCOP search set to: ", self.SCOP
         if env_flag == 'SSMSEARCH':
            self.SSM=False
            print "SSM search set to: ", self.SSM
         if env_flag == 'PQSSEARCH':
            self.PQS=False
            print "PQS search set to: ", self.PQS
         if env_flag == 'FASTALOCAL':
            self.fasta_local=False
            print "Run local Fasta search set to: ", self.fasta_local
         if env_flag == 'RUNLOCAL':
            self.run_local=False
            print "Run local set to: ", self.run_local
         if env_flag == 'ONLYMODELS':
            # If HKLIN and SEQIN where only specified on the command line ONLYMODELS will always be true
            if self.ONLYMODELS==True:
               self.ONLYMODELS=True
            else:
               self.ONLYMODELS=False
            print "Only generate models set to: ", self.ONLYMODELS
         if env_flag == 'UPDATE':
            self.UPDATE=False
            print "Update database files set to: ", self.UPDATE
         if env_flag == 'ISOLATED':
            self.ISOLATED=False
            print "Run in ISOLATED mode set to: ", self.ISOLATED
         if env_flag == 'MDLDPDBCLP':
            self.MDLDPDBCLP=False
            print "Generate PDBClip models set to: ", self.MDLDPDBCLP
         if env_flag == 'MDLPLYALA':
            self.MDLPLYALA=False
            print "Generate Polyalanine models set to: ", self.MDLPLYALA
         if env_flag == 'MDLMOLREP':
            self.MDLMOLREP=False
            print "Generate Molrep models set to: ", self.MDLMOLREP
         if env_flag == 'MDLCHAINSAW':
            self.MDLCHAINSAW=False
            print "Generate Chainsaw models set to: ", self.MDLCHAINSAW

   def parse_LABIN(self, data_line):
      """ A function to parse the LABIN keyword. """

      labels=[]

      # Check that the correct number of arguments are given
      if data_line.count('=') != 3:
         self.keyword_usage('labin', 'Incorrect number of arguments for LABIN', 1)

      # Split the line into its seperate parts
      line=string.split(data_line,'=')

      # Remove the 'LABIN' word from the start of the line
      labin_part=string.split(line[0])
      del labin_part[0]

      # Remove the first entry and re-enter it without 'LABIN'
      del line[0]
      line.insert(0,labin_part[0])

      # Now split the line again
      for i in line:
        t=string.split(i)
        for j in t:
           labels.append(j)

      # Assign the column labels from the list (0=1, 2=3 4=5)
      self.col_labels[labels[0].upper()]=labels[1]
      self.col_labels[labels[2].upper()]=labels[3]
      self.col_labels[labels[4].upper()]=labels[5]

      # Check that all of the necessary column labels are given
      print ""
      for i in 'F', 'SIGF' , 'FREER_FLAG':
         if i not in self.col_labels.keys(): 
            self.keyword_usage('labin', "Column label %s not given" % i, 2)
         print "column %s labelled as %s" % (i, self.col_labels[i]) 
      print ""

   def verify_NUM(self):
      """ A function to chekc that the NUM_ variables adhere to the constraints. """

      # Verify that ENSEMNUM is less than MRNUM
      if self.ENSEMNUM > self.MRNUM:
         self.keyword_usage('NUM_values', 'ENSEMNUM (%d) should be less than or equal to MRNUM (%d)' % (self.ENSEMNUM, self.MRNUM), 9)

   def keyword_usage(self, keyword, error_string, error_code):
      """ A function to echo the correct usage of the keywords to the stdout. """

      self.setErrorString(error_string)
 
      print "Keyword error:"

      # LABIN errors:
      if keyword == 'labin':
         print "LABIN error: %s" % self.error_string
         print "Keyword LABIN must be provided and must specify 3 column labels: F, SIGF and FreeR_flag"
         print "  e.g. LABIn F=FP SIGF=SIGFP FreeR_flag=FREE"
     
      # JOBID errors
      if keyword == 'jobid':
         print "JOBID error: %s" % self.error_string
         print "Keyword JOBID must be provided"
         print "  e.g. JOBID Test_job1"
     
      # ROOTDIR errors
      if keyword == 'root_dir':
         print "ROOTDIR error: %s" % self.error_string
     
      # ROOTDIR errors
      if keyword == 'reshtml':
         print "RESHTML error: %s" % self.error_string
     
      # IGNORE PDB id errors
      if keyword == 'ignore':
         print "IGNORE error: %s" % self.error_string
         print "Keyword IGNORE must specify PDB codes"
         print "  e.g. IGNORE 1smw 1smm 1smu 1ahc ......"

      # INCLUDE PDB id errors
      if keyword == 'include':
         print "INCLUDE error: %s" % self.error_string
         print "Keyword INCLUDE must specify chain codes. This is a four character PDB code\n" \
               + "followed by an underscore and a chain identifier letter."
         print "  e.g. INCLUDE 1smw_A 1smm_B 1smu_A 1ahc_C ......"

      # NMASU errors
      if keyword == 'NMASU':
         print "%s error: %s" % (keyword, self.error_string)
         print "  e.g. %s 4" % keyword
    
      # PACK errors
      if keyword == 'PACK':
         print "%s error: %s" % (keyword, self.error_string)
         print "  e.g. %s 4" % keyword
    
      # NCYC errors
      if keyword == 'NCYC':
         print "%s error: %s" % (keyword, self.error_string)
         print "  e.g. %s 10" % keyword
    
      # NUM_values errors
      if keyword == 'NUM_values':
         print "'NUM' variables error: %s" % self.error_string

      if keyword == 'MRNUM' or keyword == 'ENSEMNUM':
         print "%s error: %s" % (keyword, self.error_string)
         print "  e.g. %s 20" % keyword
    
      # EVALUE errors
      if keyword == 'E_value':
         print "EVALUE error: %s" % self.error_string
         print "  e.g. EVALUE 0.02"

      # Debug errors
      if keyword == 'DEBUG' or keyword == 'CLUSTER' or keyword == 'SCOPSEARCH' or keyword == 'SSMSEARCH' \
            or keyword == 'PQSSEARCH' or keyword == 'PWALIGN' or keyword == 'FASTALOCAL' \
            or keyword == 'RUNLOCAL' or keyword == 'ONLYMODELS' or keyword == 'UPDATE' \
            or keyword == 'ISOLATED' or keyword == 'MDLCHAINSAW' or keyword == 'MDLMOLREP' \
            or keyword == 'MDLPLYALA' or keyword == 'MDLDPDBCLP':  
         print "%s error: %s" % (keyword, self.error_string)
         print "  e.g. %s True" % keyword

      # MRPROGRAM errors
      if keyword == 'MRPROGRAM':
         print "MRPROGRAM error: %s" % self.error_string
         print "Keyword MRPROGRAM must be 'MOLREP' or 'PHASER' or 'AMORE'"
         print "  e.g. MRPROGRAM PHASER"

      # MAPROGRAM errors
      if keyword == 'MAPROGRAM':
         print "MAPROGRAM error: %s" % self.error_string
         print "Keyword MAPROGRAM must be 'MAFFT' or 'CLUSTALW'"
         print "  e.g. MAPROGRAM Mafft"

      sys.exit(error_code)

if __name__ == '__main__':

   k=Keywords()
   k.parse_LABIN("LABIn FP=FP SIGF=SIGFP freer_flag=FREE")

