#!/usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#

import os, sys, string, time
import shutil

import Matches, MRBUMP_target_info, Model
import Model_Queue
import Model_struct, Ensemble
import PQS
import MRBUMP_MR
import Cleanup
import MRBUMP_loggraphs


class Master:
   """ Master script for running MRBUMP """
 
   def __init__(self):
      self.name = ''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False
 
   def setDEBUG(self, flag):
      self.debug=flag  

   def fast_search_MR(self, init):

      # Create an instance of mstat
      mstat=Matches.Match_struct()
      mstat.setSearchDir(init.search_dir)
      mstat.setResultsDir(init.results_dir)
      mstat.setScratchDir(init.scratch_dir)

      # Target sequence information
      sys.stdout.write("\n")
      sys.stdout.write("###################################\n")
      sys.stdout.write("###      Target Information     ###\n")
      sys.stdout.write("###################################\n")
      sys.stdout.write("\n")

      target_info=MRBUMP_target_info.TargetInfo()
      mstat.InitError = target_info.setTargetInfo(init.seqin, init.hklin, mstat.search_dir, \
         init.keywords.col_labels, init.keywords.job_ID)
      target_info.printTargetInfo()

      # Update the no. of mols in the a.s.u. if the user has specified a value
      if init.keywords.NMASU != None:
         print "The estimated number of molecules in the asu is: %d" % target_info.no_of_mols 
         print "The user has specified a value of %d" % init.keywords.NMASU 
         print "The value will be set to the user defined value"
         mstat.setEstNoofMols(target_info.no_of_mols)
         target_info.no_of_mols = init.keywords.NMASU

      # Set the DB's for local-based searches
      mstat.setFastaDB(init.mrbump + '/data/pdb_ATOMseqs.txt')

      # Set the output results html page
      mstat.setResultsHTML(init.results_html)

      # Make a note of the start time for the job
      mstat.setStartTime(time.time())

      # Set the model preparation and MR parameters
      mstat.setMRNUM(init.keywords.MRNUM)
      mstat.setENSEMNUM(init.keywords.ENSEMNUM)
      mstat.setMRPROGRAM((init.keywords.MRPROGRAM).upper())

      # initialize the results page
      #w=Write_Search_results.Write_html()
      mstat.search_results.write_header(init, mstat.results_html)
      mstat.search_results.write_footer(init, mstat, mstat.results_html)

      # write the target details
      mstat.search_results.write_target_details(init, mstat, target_info, mstat.results_html)
      
      # If there was an error in the initialisation exit the program
      if mstat.job_end == True:
         sys.exit()
      
      # Search information
      sys.stdout.write("\n")
      sys.stdout.write("###################################\n")
      sys.stdout.write("###         Fasta search        ###\n")
      sys.stdout.write("###################################\n")
      sys.stdout.write("\n")

      seq_search=Matches.SEQ_match()

      seq_search.get_Fasta_file(init, mstat, target_info, mstat.results_dir + '/data/matches', \
         init.environment.SETUP_DIR)

      # Assign the ignore list, i.e. the PDB codes that are to be ignored
      mstat.ignore_list=init.keywords.ignore_list

      # Assign the include list, i.e. the CHAIN codes that are to be included
      mstat.include_list=init.keywords.include_list

      seq_search.get_Fasta_matches(init, mstat)

      # Exit and finish html if there were no matches
      if mstat.no_of_OCAhits == 0:
         mstat.search_results.write_SEQ_search_details(mstat, mstat.results_html)
         sys.exit()

      ps=Matches.Process_Score()

      sys.stdout.write("\n")
      sys.stdout.write("---------------------------------------------------------\n")
      sys.stdout.write("Fasta: Getting Template Sequences and Multiple Alignment:\n")
      sys.stdout.write("---------------------------------------------------------\n")
      sys.stdout.write("\n")

      #ps.PDB_process(mstat, 'OCA')

      # Extract the sequences for the chains from the pdb datafile
      ps.getPDB_sequences(mstat, "OCA")

      # Set the names of the input and output files to be used in the multiple alignment 
      mstat.setAllRawFile(mstat.search_dir + "/sequences/All_raw_seqs_OCA.fasta")
      mstat.setAllAlignFile(mstat.search_dir + "/sequences/All_align_seqs_OCA.fasta")

      # Do the multiple alignment
      ps.alignment(mstat.chain_list.keys(), target_info, init, mstat)

      # Get the top hit and sort the list of aligned sequences
      ps.get_top_hit(mstat.chain_list.keys(), mstat)

      # write the OCA search details to the html page
      mstat.search_results.write_SEQ_search_details(init, mstat, mstat.results_html)

      #shutil.copy('/tmp/ssm_result.xml', mrsearchdir + '/results/ssm_result.xml')

      if init.keywords.SSM == True:

         sys.stdout.write("\n")
         sys.stdout.write("###################################\n")
         sys.stdout.write("###          SSM search         ###\n")
         sys.stdout.write("###################################\n")
         sys.stdout.write("\n")

         ssm_search=Matches.SSM_match()

         ssm_search.setSSMScript(init.mrbump_incl + '/perl/ssm.pl')
         ssm_search.get_SSM_file(mstat, init.keywords.job_ID, mstat.results_dir + '/data/ssm_result.xml')
         ssm_search.get_SSM_matches(mstat)

         sys.stdout.write("\n")
         sys.stdout.write("-------------------------------------------------------\n")
         sys.stdout.write("SSM: Getting Template Sequences and Multiple Alignment:\n")
         sys.stdout.write("-------------------------------------------------------\n")
         sys.stdout.write("\n")

         #ps.PDB_process(mstat, 'SSM')

         # Extract the sequences for the chains from the pdb datafile
         ps.getPDB_sequences(mstat, "SSM")
   
         # Set the names of the input and output files to be used in the multiple alignment 
         mstat.setAllRawFile(mstat.search_dir + "/sequences/All_raw_seqs_SSM.fasta")
         mstat.setAllAlignFile(mstat.search_dir + "/sequences/All_align_seqs_SSM.fasta")
   
         # Do the multiple alignment
         ps.alignment(mstat.chain_list.keys(), target_info, init, mstat)

         # Get the top hit and sort the list of aligned sequences
         ps.get_top_hit(mstat.chain_list.keys(), mstat)

         # write the SSM search details to the html page
         mstat.search_results.write_SSM_search_details(init, mstat, mstat.results_html)
 
      # Prune the list
      ps.prune_list(mstat)
      ps.print_list(mstat)
      
      #Ouput the loggraph data for the alignment results
      lg_plot=MRBUMP_loggraphs.Mrbump_logs()
      lg_plot.plot_alignment_graphs(init, mstat)
  
      ps.PDB_process(init, mstat)
 
      # Search the SCOP database to see if any domains exist in the template structures

      if init.keywords.SCOP == True:

         sys.stdout.write("\n")
         sys.stdout.write("#####################################################\n")
         sys.stdout.write("###          Searching for Domains (SCOP)         ###\n")
         sys.stdout.write("#####################################################\n")
         sys.stdout.write("\n")

         scop_search=Matches.SCOP_match()
         scop_search.get_SCOP_matches(mstat, init.environment.SETUP_DIR)
   
         sys.stdout.write("\n")
         sys.stdout.write("---------------------------\n")
         sys.stdout.write("SCOP: Processing PDB files:\n")
         sys.stdout.write("---------------------------\n")
         sys.stdout.write("\n")

         # ps.PDB_process(mstat, 'SCOP')

         # Extract the sequences for the chains from the pdb datafile
         #ps.getPDB_sequences(mstat, "SSM")
   
         # Set the names of the input and output files to be used in the multiple alignment 
         mstat.setAllRawFile(mstat.search_dir + "/sequences/All_raw_seqs_SCOP.fasta")
         mstat.setAllAlignFile(mstat.search_dir + "/sequences/All_align_seqs_SCOP.fasta")

         # Add the SCOP results into the MR list before re-alignment and sorting   
         mstat.sorted_MR_list = mstat.sorted_MR_list + mstat.Domains_list

         # Do the multiple alignment
         ps.alignment(mstat.sorted_MR_list, target_info, init, mstat)

         # Get the top hit and sort the list of aligned sequences
         ps.get_top_hit(mstat.sorted_MR_list, mstat)

         # write the SSM search details to the html page
         mstat.search_results.write_domains_search_details(init, mstat, mstat.results_html)
 
         # Re-sort the list to take into account the SCOP results scores
         ps.re_sort_MR_list(mstat)
         ps.print_pruned_list(mstat)

      # write the overall search details to the html page
      mstat.search_results.write_ALL_search_details(init, mstat, target_info, mstat.results_html)
       
      # Do the modelling of the PDB model structures
      
      sys.stdout.write("\n")
      sys.stdout.write("###########################################\n")
      sys.stdout.write("###       Search Model Preperation      ###\n")
      sys.stdout.write("###########################################\n")
      sys.stdout.write("\n")
      
      # Output the details of what will be modelled
      sys.stdout.write("The following types of model will be generated:\n")
      sys.stdout.write("\n")
      for i in init.model_types:
         sys.stdout.write("%s -- " % i)
         if i == 'PDBCLP':
            sys.stdout.write("PDBclip model, waters, hydrogens removed + most probable side-chain conformations.\n") 
         if i == 'PLYALA':
            sys.stdout.write("Polyalanine model, all side chains removed.\n") 
         if i == 'MOLREP':
            sys.stdout.write("Molrep generated model, side chains pruned according to sequence alignment.\n") 
         if i == 'CHNSAW':
            sys.stdout.write("Chainsaw generated model, side chains pruned according to sequence alignment.\n") 
      sys.stdout.write("\n")

      model=Model.Modelling()
      model.convert_to_pir(mstat, target_info)

      # Prepare polyalanine models using 
      if init.keywords.MDLPLYALA:
         model.polyala(target_info, init, mstat)

      # Prepare models using Molrep
      if init.keywords.MDLMOLREP:
         model.molrep(target_info, init, mstat)
      
      # Multiple alignment input to Chainsaw
      if init.keywords.MDLCHAINSAW:
         model.chainsaw(target_info, init, mstat)
      
      # Monitor the queue for the completion of all the jobs
      
      if init.keywords.CLUSTER == True:
         queue=Model_Queue.Queue_info()
         queue.qstat_check(init, mstat, target_info, 'MODEL_PREP')
         mstat.resetJobid_Array()
         mstat.resetJobid_Dict()
         mstat.resetJobName_Dict()
   
      # write the overall search details to the html page
      mstat.search_results.write_model_prep_details(init, mstat, target_info, mstat.results_html)
      
      # Ouput the loggraph data for the model preparation results
      if init.keywords.MDLMOLREP or init.keywords.MDLCHAINSAW:
         lg_plot=MRBUMP_loggraphs.Mrbump_logs()
         lg_plot.plot_model_prep_graphs(init, mstat)
  
      #If user only wants to generate the models then we will exit at this point.
      if init.keywords.ONLYMODELS == True:
         finish_time = time.ctime()
         finish_file = open(mstat.results_dir + '/finished.txt', 'w')
         finish_file.write('Job finished: ' + finish_time)
         finish_file.write('')
         finish_file.write('Information only, please do not reply to this email.')
         finish_file.close()
      
         # Output the list of models that has been created
         mr_setup=Model_struct.MR_setup()
         mr_setup.setTypes(init, mstat, mstat.search_dir)

         # Print the references and exit
         mstat.mr_results.print_references(init, mstat)
         mstat.mr_results.print_references_stdout(init)
         sys.exit(0)      

      # Setup the models for MR
      sys.stdout.write("\n")
      sys.stdout.write("##############################################################\n")
      sys.stdout.write("###           Molecular Replacement and Refinement         ###\n")
      sys.stdout.write("##############################################################\n")
      sys.stdout.write("\n")
      
      mr_setup=Model_struct.MR_setup()
      mr_setup.setTypes(init, mstat, mstat.search_dir)
      
      # Write the header for the MR secion
      mstat.mr_results.write_MR_header(init, mstat)
      
      # If the number of molecules in our target is greater than 1 consult PQS for multimers
      
      if target_info.no_of_mols >= 2 and init.keywords.PQS == True:
      
         print ""
         print "------------------------------"
         print "Searching for Multimers (PQS):"
         print "------------------------------"
         print ""
      
         pqs=PQS.PQS()
      
         pqs.set_pqs_list(init.environment.SETUP_DIR)
         pqs.get_pqs_files(target_info, mstat, init.keywords.MAPROGRAM, mstat.search_dir)
      
         # write the PQS search details to the html page
         mstat.search_results.write_PQS_search_details(init, mstat, target_info, mstat.results_html)
      
      # If we are using phaser we need to create an ensemble from the top search models
      if mstat.MRPROGRAM == "PHASER":  

         print ""
         print "--------------------------------"
         print "Setting up ensemble for Phaser:"
         print "--------------------------------"
         print ""
      
         # Make sure we are not asking for more models than are in the MR list
         if mstat.num_MR_models < init.keywords.ENSEMNUM:
	    init.keywords.ENSEMNUM=mstat.num_MR_models
	    sys.stdout.write("Resetting the number in the ensmemble to " + `init.keywords.ENSEMNUM` 
	                     + " as there are not enough models for the specified value\n")
	    sys.stdout.write("\n")

         ens=Ensemble.Ensemble()
         ens.setup_ensemble(mstat, init.keywords.ENSEMNUM)
      
         # write the ensemble details to the html page
         mstat.search_results.write_ensemble_details(init, mstat, target_info, mstat.results_html)
      
      # Do the molecular replacement using the model list
      MR=MRBUMP_MR.MR_submit()
      #MR.MR_setup(target_info, init, mstat)
      MR.MR_start(target_info, init, mstat)
      
      # Poll the queue to see if the jobs have finished
      if init.keywords.CLUSTER == True:
      
         print ""
         print "---------------------------"
         print "Results (post-refinement):"
         print "---------------------------"
         print ""
           
         queue.qstat_check(init, mstat, target_info, 'MOL_REPLACE')
         mstat.resetJobid_Array()
         mstat.resetJobid_Dict()
         mstat.resetJobName_Dict()
      
      # If Amore failed to produce a solution we will try Molrep on each of the models
      
      if mstat.solution_found == False and mstat.MRPROGRAM == 'AMORE':
      
         mstat.setMRPROGRAM('MOLREP')
      
         MR.MR_start(target_info, init, mstat)
      
         if init.keywords.CLUSTER == True:
      
            print ""
            print "---------------------------"
            print "Results (post-refinement):"
            print "---------------------------"
            print ""
           
            queue.qstat_check(init, mstat, target_info, 'MOL_REPLACE')
            mstat.resetJobid_Array()
            mstat.resetJobid_Dict()
            mstat.resetJobName_Dict()
      
      # If Molrep failed to produce a solution we will try Phaser on each of the models
      
      if mstat.solution_found == False and mstat.MRPROGRAM == 'MOLREP' and init.phaser_found == True:
      
         mstat.setMRPROGRAM('PHASER')
      
         print ""
         print "--------------------------------"
         print "Setting up ensemble for Phaser:"
         print "--------------------------------"
         print ""
      
         ens=Ensemble.Ensemble()
         ens.setup_ensemble(mstat, init.keywords.ENSEMNUM)
      
         # write the ensemble details to the html page
         mstat.search_results.write_ensemble_details(init, mstat, target_info, mstat.results_html)
      
         MR.MR_start(target_info, init, mstat)
      
         if init.keywords.CLUSTER == True:
      
            print ""
            print "---------------------------"
            print "Results (post-refinement):"
            print "---------------------------"
            print ""
           
            queue.qstat_check(init, mstat, target_info, 'MOL_REPLACE')
            mstat.resetJobid_Array()
            mstat.resetJobid_Dict()
            mstat.resetJobName_Dict()
 
      elif mstat.solution_found == False and init.phaser_found == False: 
          
          print ""
          print "Phaser will not be used for MR step as it is could not be found in the system path."
          print ""

      # write the final solution details to the html page
      mstat.mr_results.write_final_soln(init, mstat, target_info)

      # Display the references in the results html file and the output logfile.
      mstat.mr_results.print_references(init, mstat)
      mstat.mr_results.print_references_stdout(init)

      if mstat.solution_found == True:
         shutil.copy(mstat.model_list[mstat.soln_model].refmac_MTZOUTfile , init.hklout)
         shutil.copy(mstat.model_list[mstat.soln_model].refmac_PDBfile , init.xyzout)
           
      finish_time = time.ctime()
      finish_file = open(mstat.results_dir + '/finished.txt', 'w')
      finish_file.write('Job finished: ' + finish_time)
      finish_file.write('')
      finish_file.write('Information only, please do not reply to this email.')
      finish_file.close()
      
      # Remove the data directory to save memory space
      if init.keywords.DEBUG == False:
         print "Cleanup: Removing search data and scratch directory"
         cleanup=Cleanup.Clean()
         cleanup.rm_dir(mstat.search_dir + '/data')
         cleanup.rm_dir(mstat.search_dir + '/scratch')
      
      sys.exit(0)

