#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Script to do all of the modelling steps in MRBUMP. Calls chainsaw and molrep.
# Ronan Keegan 114/10/04

import os, string, sys
import shutil
import CCP4_Submit

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 must be defined'
#if not os.environ.has_key('MODINSTALL6v2'):
#    raise RuntimeError, 'MODINSTALL6v2 must be defined'


class Modelling:

   def __init__(self):
      if os.name == "nt":
         self.MolrepEXE=os.environ["CBIN"] + "/molrep.exe"
      else:
         self.MolrepEXE=os.environ["CBIN"] + "/molrep"
      self.ChainsawEXE=os.environ["CBIN"] + "/chainsaw"

      self.chainsaw_target_seq=""
      self.chainsaw_template_seq=""

      self.job_logfile=""
   
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag

   def setJobLogfile(self, filename):
      self.job_logfile=filename

   def convert_to_pir(self, mstat, target_info):
      """ Function to write convert alignment files into PIR format for Chainsaw. """
      
      # Temporary flag to mark whether or not to use the origional alignment
      # or the stripped one where only the target lenght is considered.
      MODIFIED = False

      print "\n---------------------------------------------------------------"
      print "Models Construct log message: Creating PIR alignment files"
      print "----------------------------------------------------------------\n"
      
      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:

         # Set the pir file variable
         mstat.chain_list[chain].setMAPIRFile(mstat.chain_list[chain].working_DIR + \
          '/alignments/ma_' + mstat.chain_list[chain].chainName + '.pir')

         if MODIFIED == True:
            # Assign output files
            outfile=open(mstat.chain_list[chain].ma_pir_file,'w')

            outfile.write('>P1;' + target_info.chainID + '\n')
            outfile.write('\n')
            outfile.write(mstat.chain_list[chain].alignment.align_tar_seq + '\n')
            outfile.write('*\n')
            outfile.write('>P1;' + mstat.chain_list[chain].chainName + '\n')
            outfile.write('\n')
            outfile.write(mstat.chain_list[chain].alignment.align_seq + '\n')
            outfile.write('*\n')
 
            outfile.close()
         else:
            # Change the unmodified alignment file to PIR format  
            algn_tmp_file = mstat.chain_list[chain].working_DIR + '/alignments/' + chain + '_temp_aln.fasta'
 
            infile=open(algn_tmp_file, 'r')
            outfile=open(mstat.chain_list[chain].ma_pir_file,'w')
         
            line=infile.readline()
            # Write the target header
            outfile.write(line.replace('>','>P1;'))
            outfile.write('\n')

            # Get the target sequence and write it to the outfile
            line=infile.readline()
            while line:
               if '>' not in line:
                  outfile.write(line)
               else:
                  break
               line=infile.readline()
            outfile.write('*\n')

            # Write the template header
            outfile.write(line.replace('>','>P1;'))
            outfile.write('\n')

            # Get the template sequence and write it to the outfile
            line=infile.readline()
            while line:
               outfile.write(line)
               line=infile.readline()
            outfile.write('*\n')

            # Close both files
            infile.close()
            outfile.close()

   def polyala(self, target_info, init, mstat):
      """ Generate models that are polyalanine. """
 
      print "\n-------------------------------------------------------------"
      print "Models Construct log message: Preparing Polyalanine models."
      print "-------------------------------------------------------------\n"

      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:
       
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/plyala')
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/plyala/mr')
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/plyala/refine')

         # Set the output PDB model
         mstat.chain_list[chain].setPlyalaModelPDB(mstat.chain_list[chain].working_DIR + '/plyala/MODEL_PLYALA_' \
            + mstat.chain_list[chain].chainName + '.pdb')

         # Set the job logfile
         self.setJobLogfile(mstat.chain_list[chain].working_DIR + '/plyala/job.log')

         # Set the Polyala log file
         mstat.chain_list[chain].setPlyalaLogFile(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName \
            + '/plyala/MODEL_PLYALA_' + mstat.chain_list[chain].chainName + '.log')

	 # Set the job name for the Batch queue
         mstat.chain_list[chain].setPlyalaJobName("ply_" + mstat.chain_list[chain].chainName + "_" + target_info.chainID)

         if self.debug == True:
            print "Polyala prep log: Modifying template " + chain 
            print "Polyala prep log: Output PDB file:\n  " + mstat.chain_list[chain].plyala_modelPDB 
            print "Polyala prep log: Log file:\n  " + mstat.chain_list[chain].plyala_logfile + "\n" 

         # Setup the job
         plyala_job=CCP4_Submit.CCP4_sub()
         plyala_job.setProgram("pdbset")
         plyala_job.setProgramLogfile(mstat.chain_list[chain].plyala_logfile)
         plyala_job.setCommandLine("XYZIN " + mstat.chain_list[chain].PDBFile + " XYZOUT " + mstat.chain_list[chain].plyala_modelPDB)

         # Setup the keywords for the job
         plyala_job.setKEYWORD("EXCLUDE SIDE")
         plyala_job.setKEYWORD("END")

         if init.keywords.CLUSTER:
            # Submit the job to a batch queue 
            plyala_job.submit_job(mstat.chain_list[chain].plyala_jobname,\
                                  self.job_logfile, \
                                  mstat.chain_list[chain].working_DIR + "/plyala",\
                                  "SGE")
            # Not the job number and the jobname
            mstat.chain_list[chain].setPlyala_jobid(plyala_job.job_number)
            mstat.jobid_array.add(mstat.chain_list[chain].plyala_jobid)
            mstat.jobid_dict[mstat.chain_list[chain].plyala_jobid]=mstat.chain_list[chain].plyala_jobname
         else:
            # Run the job locally
            plyala_job.submit_job(mstat.chain_list[chain].plyala_jobname,\
                                  self.job_logfile, \
                                  mstat.chain_list[chain].working_DIR + "/plyala")

         mstat.model_counter=mstat.model_counter+1
                   
   def molrep(self, target_info, init, mstat):
      """ A wrapper for calling molrep to MODEL the pdb files. """
 
      print "\n-------------------------------------------------------------"
      print "Models Construct log message: Manipulating models with Molrep"
      print "-------------------------------------------------------------\n"

      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:
       
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/molrep')
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/molrep/mr')
         os.mkdir(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/molrep/refine')

         # Set the output PDB model
         mstat.chain_list[chain].setMolrepModelPDB(mstat.chain_list[chain].working_DIR + '/molrep/MODEL_MOLREP_' \
            + mstat.chain_list[chain].chainName + '.pdb')
   
         # Set the job logfile
         self.setJobLogfile(mstat.chain_list[chain].working_DIR + '/molrep/job.log')

         # Set the Molrep log file
         mstat.chain_list[chain].setMolrepLogFile(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName \
            + '/molrep/MODEL_MOLREP_' + mstat.chain_list[chain].chainName + '.log')

	 # Set the job name for the Batch queue
         mstat.chain_list[chain].setMolrepJobName("mol_" + mstat.chain_list[chain].chainName + "_" + target_info.chainID)

         if self.debug == True:
            print "Molrep prep log: Modifying template " + chain 
            print "Molrep prep log: Output PDB file:\n  " + mstat.chain_list[chain].molrep_modelPDB 
            print "Molrep prep log: Log file:\n  " + mstat.chain_list[chain].molrep_logfile + "\n" 

         # Setup the job
         molrep_job=CCP4_Submit.CCP4_sub()
         if os.name == "nt":
            molrep_job.setProgram("molrep.exe")
         else:
            molrep_job.setProgram("molrep")
         molrep_job.setProgramLogfile(mstat.chain_list[chain].molrep_logfile)
         molrep_job.setCommandLine("MODEL " + mstat.chain_list[chain].PDBFile)

         # Setup the keywords for the job
         molrep_job.setKEYWORD("DOC Y")
         molrep_job.setKEYWORD("FILE_S " +  target_info.seqfile)
         molrep_job.setKEYWORD("SURF Y")
         molrep_job.setKEYWORD("END")

         # Add a job command to move the align.pdb file to a job related name
         if os.name == "nt":
            molrep_job.addProgram("move")
         else:
            molrep_job.addProgram("mv")
         molrep_job.setProgramLogfile("mv.log")
         molrep_job.setCommandLine("align.pdb " + mstat.chain_list[chain].molrep_modelPDB) 

         if init.keywords.CLUSTER:
            # Submit the job to a batch queue 
            molrep_job.submit_job(mstat.chain_list[chain].molrep_jobname,\
                                  self.job_logfile,\
                                  mstat.chain_list[chain].working_DIR + "/molrep",\
                                  "SGE")
            # Not the job number and the jobname
            mstat.chain_list[chain].setMolrep_jobid(molrep_job.job_number)
            mstat.jobid_array.add(mstat.chain_list[chain].molrep_jobid)
            mstat.jobid_dict[mstat.chain_list[chain].molrep_jobid]=mstat.chain_list[chain].molrep_jobname
         else:
            # Run the job locally
            molrep_job.submit_job(mstat.chain_list[chain].molrep_jobname,\
                                  self.job_logfile,\
                                  mstat.chain_list[chain].working_DIR + "/molrep")

         mstat.model_counter=mstat.model_counter+1
                   
   def chainsaw(self, target_info, init, mstat):
      """ A wrapper for calling CHAINSAW to manipulate the pdb files. """ 
 
      print "\n--------------------------------------------------------------------"
      print "Models Construct log message: Manipulating templates with Chainsaw"
      print "--------------------------------------------------------------------\n"

      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:

	 # Set various parameters for the job including the job name for the SGE queue
         chainsaw_DIR=mstat.chain_list[chain].working_DIR + '/chainsaw'

         # Set the job logfile
         self.setJobLogfile(chainsaw_DIR + '/job.log')

         # Set the output chainsaw Model name
         mstat.chain_list[chain].setChainsawModelPDB(chainsaw_DIR + '/chainsaw_' \
            + mstat.chain_list[chain].chainName + '.pdb')

         # Set the Chainsaw job name
         mstat.chain_list[chain].setChainsawJobName("chain_" + mstat.chain_list[chain].chainName + "_" + target_info.chainID)
                   
         #Set the Chainsaw job logfile
         mstat.chain_list[chain].setChainsawLogFile(chainsaw_DIR + "/CHAINSAW_" + mstat.chain_list[chain].chainName + ".log")
        
         # Make the necessary directories
         os.mkdir(chainsaw_DIR)
         os.mkdir(chainsaw_DIR + '/mr')
         os.mkdir(chainsaw_DIR + '/refine')

         # Write out the correct input alignment file for chainsaw. This is with "-" doubles removed.
         chainsaw_in_aln_file=chainsaw_DIR + "/chainsaw_alignment_file_" + mstat.chain_list[chain].chainName + ".pir"
         csaw_aln_file=open(chainsaw_in_aln_file, "w") 
         csaw_aln_file.write(">P1;Target\n")
         csaw_aln_file.write("\n")
         csaw_aln_file.write(mstat.chain_list[chain].alignment.chainsaw_target_seq + "\n")
         csaw_aln_file.write("*\n")
         csaw_aln_file.write(">P1;" + mstat.chain_list[chain].chainName + "\n")
         csaw_aln_file.write("\n")
         csaw_aln_file.write(mstat.chain_list[chain].alignment.chainsaw_template_seq + "\n")
         csaw_aln_file.write("*\n")
         csaw_aln_file.close()
  
         if self.debug == True:
            print "Chainsaw (multiple) prep log: Modifying template " + chain 
            print "Chainsaw (multiple) prep log: Input alignment file:\n " + chainsaw_in_aln_file 
            print "Chainsaw (multiple) prep log: Input PDB file:\n " + mstat.chain_list[chain].PDBFile 
            print "Chainsaw (multiple) prep log: Output PDB file:\n  " + mstat.chain_list[chain].chainsaw_modelPDB 
            print "Chainsaw (multiple) prep log: Log file:\n  " + mstat.chain_list[chain].chainsaw_logfile + "\n" 

         # Setup the job
         chainsaw_job=CCP4_Submit.CCP4_sub()
         chainsaw_job.setProgram("chainsaw")
         chainsaw_job.setProgramLogfile(mstat.chain_list[chain].chainsaw_logfile)
         chainsaw_job.setCommandLine("xyzin " + mstat.chain_list[chain].PDBFile + " ALIGNIN " \
            + chainsaw_in_aln_file + " xyzout "  + mstat.chain_list[chain].chainsaw_modelPDB)

         chainsaw_job.setKEYWORD("END")

         if init.keywords.CLUSTER:
            # Submit the job to a batch queue 
            chainsaw_job.submit_job(mstat.chain_list[chain].chainsaw_jobname,\
                                  self.job_logfile,\
                                  mstat.chain_list[chain].working_DIR + "/chainsaw",\
                                  "SGE")
            # Not the job number and the jobname
            mstat.chain_list[chain].setChainsaw_jobid(chainsaw_job.job_number)
            mstat.jobid_array.add(mstat.chain_list[chain].chainsaw_jobid)
            mstat.jobid_dict[mstat.chain_list[chain].chainsaw_jobid]=mstat.chain_list[chain].chainsaw_jobname
         else:
            # Run the job locally
            chainsaw_job.submit_job(mstat.chain_list[chain].chainsaw_jobname,\
                                    self.job_logfile,\
                                    mstat.chain_list[chain].working_DIR + "/chainsaw")

         mstat.model_counter=mstat.model_counter+1

