#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Write the Molecular Replacement results to the results.html file.
# Ronan Keegan 25.8.05 

import os, sys, string
import time, shutil

import Write_Search_results 

class Write_MR_Results:

   def __init__(self):
      self.name = ''
      self.marginal_found = False
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def write_MR_header(self, init, mstat):
      """ A functio to write the header for the MR section of the webpage. """

      line = '<br>\n'
      line += '<b><font size=6 color=red>Molecular Replacement and Refinement:</font></b><br>\n'
      line += '<br>\n'

      # Remove the footer from the resuls page
      page=Write_Search_results.Write_html()     
      page.remove_footer(mstat, mstat.results_html) 

      # Append the results to the results page
      res_html=open(mstat.results_html, 'a')
      res_html.write(line)
      res_html.close()

      # Add the footer to the results page
      page.write_footer(init, mstat, mstat.results_html)

   def write_marginal_soln(self, init, mstat, model, final_rfree, diff):
      """ A function to write out a marginal solution result to the results HTML file. """
      
      # Copy the results files to the solutions folder
       
      self.marginal_found = True
      model_dir_name = model + "_" + mstat.MRPROGRAM
      os.mkdir(mstat.results_dir + '/marginal_solns/' + model_dir_name)

      shutil.copytree(mstat.model_list[model].model_directory + '/refine', mstat.results_dir + '/marginal_solns/' + model_dir_name + '/refine')
 
      # Point the file links to this directory
 
      directory = init.relative_link + 'marginal_solns/' + model_dir_name + '/refine' 

      logfile_name = string.split(mstat.model_list[model].refmac_logfile,'/')[-1] 
      logfile = directory + '/' + logfile_name 
      
      mtzfile_name = string.split(mstat.model_list[model].refmac_MTZOUTfile,'/')[-1] 
      mtzfile = directory + '/' + mtzfile_name 

      pdbfile_name = string.split(mstat.model_list[model].refmac_PDBfile,'/')[-1] 
      pdbfile = directory + '/' + pdbfile_name 

      line =  '<br>\n'
      line += '<b>Marginal solution found using <b>%s</b> for the MR:</b><br><br>\n' % mstat.MRPROGRAM
      line += '<i>Template Model: %s has produced a marginal solution, final Rfree = %s and is %s of initial Rfree</i><br>\n' \
          % (model, final_rfree, diff)
      line += '<a href="' + pdbfile + '">PDB file</a> - refined template PDB file<br>\n'
      line += '<a href="' + mtzfile + '">MTZ file</a> - updated target MTZ file<br>\n'
      line += '<a href="' + logfile + '">Log file</a> - Refmac log file<br>\n'
      
      # Remove the footer from the resuls page
      page=Write_Search_results.Write_html()     
      page.remove_footer(mstat, mstat.results_html) 

      # Append the results to the results page
      res_html=open(mstat.results_html, 'a')
      res_html.write(line)
      res_html.close()

      # Add the footer to the results page
      page.write_footer(init, mstat, mstat.results_html)

      
   def write_final_soln(self, init, mstat, target_info):
      """ A function to write out the final MR solution details to the results.html page. """ 

      # Title for this section of the results.html
      line =  '<br>\n'
      line += '<b><font size=5 color=green>Final Solution:</font></b><br>\n'
      line += '<br>\n'
 
      if mstat.solution_found == True:
         SOLN_MODEL=mstat.model_list[mstat.soln_model]
 
         #Reset the output files to the ones stored in the results/solution folder.
         if SOLN_MODEL.MR_program == 'Molrep':
            soln_molrep_PDBfile = init.relative_link + 'solution/mr/' + string.split(SOLN_MODEL.molrep_PDBfile,'/')[-1]
            soln_molrep_logfile = init.relative_link + 'solution/mr/' + string.split(SOLN_MODEL.molrep_logfile,'/')[-1]
         elif SOLN_MODEL.MR_program == 'Phaser':
            soln_phaser_PDBfile = init.relative_link + 'solution/mr/' + string.split(SOLN_MODEL.phaser_PDBfile,'/')[-1]
            soln_phaser_logfile = init.relative_link + 'solution/mr/' + string.split(SOLN_MODEL.phaser_logfile,'/')[-1]
         soln_refmac_PDBfile = init.relative_link + 'solution/refine/' + string.split(SOLN_MODEL.refmac_PDBfile,'/')[-1]
         soln_refmac_MTZOUTfile = init.relative_link + 'solution/refine/' + string.split(SOLN_MODEL.refmac_MTZOUTfile,'/')[-1]
         soln_refmac_logfile = init.relative_link + 'solution/refine/' + string.split(SOLN_MODEL.refmac_logfile,'/')[-1]
 
         # Write out the name and details of the template that produced the solution 
         if SOLN_MODEL.chain_source[5] != '_':
            line += 'Chain ' + SOLN_MODEL.chain_source + ' of structure <b>' + SOLN_MODEL.PDBName + \
                 '</b> was found to be a good template model for the target structure.<br>\n'
         else:
            line += 'Structure <b>' + SOLN_MODEL.PDBName + \
                 '</b> was found to be a good template model for the target structure.<br>\n'
         line += 'The template was prepared using the ' + SOLN_MODEL.type + ' method.<br>\n'

         # Write out the Molecular Replacement results
         line += '<br>\n'
         line += '<b>Molecular Replacement</b><br><br>\n'
         line += 'Molecular Replacement program: ' + SOLN_MODEL.MR_program + '<br>\n'

         # Write the results for the MR job according to the program used
         if SOLN_MODEL.MR_program == 'Molrep':
            line += '<i>Correlation Coefficient</i>: %.3f <br>\n' % SOLN_MODEL.molrep_Corr
            line += '<i>Rfactor</i>: %.3f <br>\n' % SOLN_MODEL.molrep_Rfac
            line += '<a href="' + soln_molrep_PDBfile + '">PDB file</a> - phased PDB file<br>\n'
            line += '<a href="' + soln_molrep_logfile + '">Log file</a> - Molrep log file<br>\n'
         elif SOLN_MODEL.MR_program == 'Phaser':
            line += '<i>LLG Score</i>: %.3f <br>\n' % SOLN_MODEL.phaser_LLG_score
            line += '<i>Zscore</i>: %.3f <br>\n' % SOLN_MODEL.phaser_Zscore
            line += '<a href="' + soln_phaser_PDBfile + '">PDB file</a> - phased PDB file<br>\n'
            line += '<a href="' + soln_phaser_logfile + '">Log file</a> - Phaser log file<br>\n'

         # Write the refinement results includeing where to find the results files.
         line += '<br>\n'
         line += '<b>Refinement</b><br><br>\n'
         line += 'The template model was refined using ' + SOLN_MODEL.Refinement_program + '<br>\n'
         line += '<i>Initial Rfree</i>: %.3f <br>\n' % SOLN_MODEL.refmac_init_score
         line += '<i>Final Rfree</i>: %.3f (after 30 cycles)<br>\n' % SOLN_MODEL.refmac_finl_score
         line += '<br>\n'
         line += '<a href="' + soln_refmac_PDBfile + '">PDB file</a> - refined template PDB file<br>\n'
         line += '<a href="' + soln_refmac_MTZOUTfile + '">MTZ file</a> - updated target MTZ file<br>\n'
         line += '<a href="' + soln_refmac_logfile + '">Log file</a> - Refmac log file<br>\n'
        
      # If no solution was found just report that fact
      else:
         line +=  '<br>\n'
         line += 'No Solution found.<br>\n'
         line += '<br>\n'
         
      # Remove the footer from the resuls page
      page=Write_Search_results.Write_html()     
      page.remove_footer(mstat, mstat.results_html) 

      # Append the results to the results page
      res_html=open(mstat.results_html, 'a')
      res_html.write(line)
      res_html.close()

      # Add the footer to the results page
      mstat.setJob_End(True)
      page.write_footer(init, mstat, mstat.results_html)


   def display_results(self, mstat, target_info):
      """ A function to display results in the shell window. """
      
      len_name=len(target_info.chainID)
    
      print "" 
      print "###########################################" + "#"*len_name + "##################"
      print "#######           Solution for Target/Job: " + target_info.chainID + "          ########"
      print "###########################################" + "#"*len_name + "##################"
      print ""
       
      print 'Solution template structure: ' + mstat.model_list[mstat.soln_model].PDBName
      print 'Chain: ' + mstat.model_list[mstat.soln_model].chain_source
      print 'Model Preperation Method: ' + mstat.model_list[mstat.soln_model].type + '\n'

      print 'Molecular Replacement Method: ' + mstat.model_list[mstat.soln_model].MR_program
      if mstat.model_list[mstat.soln_model].MR_program == 'Molrep':
         print 'Phased PDB file: ' + string.split(mstat.model_list[mstat.soln_model].molrep_PDBfile,'/')[-1] + '\n'
      elif mstat.model_list[mstat.soln_model].MR_program == 'Phaser':
         print 'Phased PDB file: ' + string.split(mstat.model_list[mstat.soln_model].phaser_PDBfile,'/')[-1] + '\n'

      print 'Refinement Method: ' + mstat.model_list[mstat.soln_model].Refinement_program
      if mstat.model_list[mstat.soln_model].Refinement_program == 'Refmac':
         print 'Initial Rfree: %.3f and Rfree after 30 cycles: %.3f ' \
             % (mstat.model_list[mstat.soln_model].refmac_init_score, mstat.model_list[mstat.soln_model].refmac_finl_score)
         print 'Refined PDB file: ' + string.split(mstat.model_list[mstat.soln_model].refmac_PDBfile,'/')[-1]
         print 'Udpdated MTZ file: ' + string.split(mstat.model_list[mstat.soln_model].refmac_MTZOUTfile,'/')[-1] + '\n'
      print 'Solution files can be found in the following directory:\n %s' \
         % (mstat.results_dir + '/solution')
      print 'The results can also be viewed in HTML format in the file:\n %s' \
         % mstat.results_html
      
      print ""
      print "###########################################" + "#"*len_name + "##################"
      print ""
    
   def copy_solution_files(self, mstat, soln_dir):
      """ A function to copy the solution files to the results/solution directory"""

      if os.path.isdir(soln_dir):
         print "Solution directory already exists"
      else:
         shutil.copytree(mstat.model_list[mstat.soln_model].model_directory, soln_dir) 
    
   def print_references(self, init, mstat):
      """ A function to print the references for the programs used in the job. """

      line  = "<h3>MrBUMP Program References</h3>"

      line += "<P>"
      line += "Any publication arising from use of MrBUMP should"
      line += "include the following reference:</P>"
      line += "<P>"
      line += "R.M.Keegan and M.D.Winn (2006) <em>in preparation</em>"
      line += "</P>"
      line += "In addition, authors of specific programs should be referenced where"
      line += "applicable:"

      line += "<dl>"
      line += "<dt>CCP4"
      line += "<dd>Collaborative Computational Project, Number 4. (1994), &quot;The CCP4 Suite: Programs "
      line += "for Protein Crystallography&quot;. <em>Acta Cryst.</em> <b>D50</b>, 760-763 "
      line += "<dt>FASTA"
      line += "<dd>W. R. Pearson and D. J. Lipman (1988), &quot;Improved Tools"
      line += "for Biological Sequence Analysis&quot;, <em>PNAS</em> <b>85</b>, 2444-2448"

      if init.keywords.SSM == True:
         line += "<dt>SSM"
         line += "<dd>E.Krissinel and K.Henrick (2004), &quot;Secondary-structure matching (SSM), a new tool "
         line += "for fast protein structure alignment in three dimensions&quot;"
         line += "<em>Acta Cryst.</em> <b>D60</b>, 2256-2268"

      if init.keywords.SCOP == True:
         line += "<dt>SCOP"
         line += "<dd>A.G.Murzin, S.E.Brenner, T.Hubbard & C.Chothia (1995), <em>J.Mol.Biol.</em>,"
         line += "<b>247</b>, 536-540"

      if init.keywords.MAPROGRAM == "MAFFT":
         line += "<dt>MAFFT"
         line += "<dd>K. Katoh, K. Kuma, H. Toh and T. Miyata (2005)"
         line += "&quot;MAFFT version 5: improvement in accuracy of multiple sequence alignment&quot; "
         line += "<em>Nucleic Acids Res.</em> <b>33</b>, 511-518"

      if init.keywords.MAPROGRAM == "CLUSTALW":
         line += "<dt>CLUSTALW"
         line += "<dd>Chenna, Ramu, Sugawara, Hideaki, Koike,Tadashi, Lopez, Rodrigo, "
         line += "Gibson, Toby J, Higgins, Desmond G, Thompson, Julie D. (2003)"
         line += "&quot;Multiple sequence alignment with the Clustal series of programs&quot;"
         line += "<em>Nucleic Acids Res</em> <b>31</b>, 3497-500"

      if init.keywords.MDLCHAINSAW == True:
         line += "<dt>CHAINSAW"
         line += "<dd>N.D.Stein (2006) <em>in preparation</em>"

      if init.keywords.MDLMOLREP or init.keywords.ONLYMODELS == False:
         line += "<dt>MOLREP"
         line += "<dd>A.A.Vagin & A.Teplyakov (1997) <em>J. Appl. Cryst.</em> <b>30</b>, 1022-1025"
  
      if init.keywords.ONLYMODELS == False:
         line += "<dt>PHASER"
         line += "<dd>McCoy, A.J., Grosse-Kunstleve, R.W., Storoni, L.C. & Read, R.J. (2005). "
         line += "&quot;Likelihood-enhanced fast translation functions&quot; <i>Acta Cryst</i> <b>D61</b>, 458-464"
         line += "<dt>REFMAC"
         line += "<dd>G.N. Murshudov, A.A.Vagin and E.J.Dodson, (1997) &quot;Refinement of Macromolecular "
         line += "Structures by the Maximum-Likelihood Method&quot; <em>Acta Cryst.</em>"
         line += "<b>D53</b>, 240-255"
      line += "</dl>"

      # Remove the footer from the resuls page
      page=Write_Search_results.Write_html()     
      page.remove_footer(mstat, mstat.results_html) 

      # Append the results to the results page
      res_html=open(mstat.results_html, 'a')
      res_html.write(line)
      res_html.close()

      # Make a note of the job completion time and set the total job execution time
      mstat.setEndTime(time.time())
      total_time = mstat.end_time - mstat.start_time
   
      mstat.setJob_Time(total_time)

      # Add the footer to the results page
      mstat.setJob_End(True)
      page.write_footer(init, mstat, mstat.results_html)

   def print_references_stdout(self, init):
      """ A function to print the references to the stdout for the programs used in the job. """

      line  = "\n"
      line += "##############################################\n"
      line += "###        MrBUMP Program References       ###\n"
      line += "##############################################\n"
      line += "\n"

      line += "Any publication arising from use of MrBUMP should include the following reference:\n"
      line += "\n"
      line += "R.M.Keegan and M.D.Winn (2006) in preparation.\n"
      line += "\n"
      line += "In addition, authors of specific programs should be referenced where applicable:\n"

      line += "\n"
      line += "CCP4: Collaborative Computational Project, Number 4. (1994), 'The CCP4 Suite: Programs \n"
      line += "for Protein Crystallography'. Acta Cryst. D50, 760-763 \n"
      line += "\n"
      line += "FASTA: W. R. Pearson and D. J. Lipman (1988), 'Improved Tools\n"
      line += "for Biological Sequence Analysis', PNAS 85, 2444-2448\n"

      if init.keywords.SSM == True:
         line += "\n"
         line += "SSM: E.Krissinel and K.Henrick (2004), 'Secondary-structure matching (SSM), a new tool \n"
         line += "for fast protein structure alignment in three dimensions' Acta Cryst. D60, 2256-2268\n"

      if init.keywords.SCOP == True:
         line += "\n"
         line += "SCOP: A.G.Murzin, S.E.Brenner, T.Hubbard & C.Chothia (1995), J.Mol.Biol., 247, 536-540\n"

      if init.keywords.MAPROGRAM == "MAFFT":
         line += "\n"
         line += "MAFFT: K. Katoh, K. Kuma, H. Toh and T. Miyata (2005) 'MAFFT version 5: improvement\n"
         line += "in accuracy of multiple sequence alignment' Nucleic Acids Res. 33, 511-518\n"

      if init.keywords.MAPROGRAM == "CLUSTALW":
         line += "\n"
         line += "CLUSTALW: Chenna, Ramu, Sugawara, Hideaki, Koike,Tadashi, Lopez, Rodrigo, \n"
         line += "Gibson, Toby J, Higgins, Desmond G, Thompson, Julie D. (2003)\n"
         line += "'Multiple sequence alignment with the Clustal series of programs'\n"
         line += "Nucleic Acids Res 31, 3497-500\n"

      if init.keywords.MDLCHAINSAW == True:
         line += "\n"
         line += "CHAINSAW: N.D.Stein (2006) in preparation\n"

      if init.keywords.MDLMOLREP or init.keywords.ONLYMODELS == False:
         line += "\n"
         line += "MOLREP: A.A.Vagin & A.Teplyakov (1997) J. Appl. Cryst. 30, 1022-1025\n"

      if init.keywords.ONLYMODELS == False:
         line += "\n"
         line += "PHASER: McCoy, A.J., Grosse-Kunstleve, R.W., Storoni, L.C. & Read, R.J. (2005). \n"
         line += "'Likelihood-enhanced fast translation functions' Acta Cryst D61, 458-464\n"
         line += "\n"
         line += "REFMAC: G.N. Murshudov, A.A.Vagin and E.J.Dodson, (1997) 'Refinement of Macromolecular \n"
         line += "Structures by the Maximum-Likelihood Method' Acta Cryst. D53, 240-255\n"
         line += "\n"

      print line

