#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Wrapper for clustalw the sequence multiple alignment program
# Ronan Keegan 13/9/05

import os, sys, string


class Clustalw:
   """ A class to call clustalw to do a multiple alignment of the target and template sequences. """
 
   def __init__(self):

      self.clustalw_EXE = 'clustalw'

      self.input_fasta_file = ''
      self.output_pir_file = ''
      self.log_file = ''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def setInputFastaFile(self, file_name):
      self.input_fasta_file = file_name

   def setOutputPIRFile(self, file_name):
      self.output_pir_file = file_name

   def setLogFile(self, file_name):
      self.log_file = file_name

   def Clustalw(self, input_fasta_file, output_pir_file):
      """ A function to do a system call to Clustalw."""

      self.setInputFastaFile(input_fasta_file)
      self.setOutputPIRFile(output_pir_file)
      self.setLogFile('clustalw.log')

      if self.debug:
         print ""
         print "======================"
         print "Clustalw command line:"
         print "======================"
         print self.clustalw_EXE + ' -align -outorder=input -output=pir -infile=' + self.input_fasta_file + \
                ' -outfile=' + self.output_pir_file
         print ""

      c=os.popen(self.clustalw_EXE + ' -align -outorder=input -output=pir -infile=' + self.input_fasta_file + \
             ' -outfile=' + self.output_pir_file, 'r')
      c.close()
      
   def convert_to_fasta(self):
  
      infile=open(self.output_pir_file,'r')
      
      line=infile.readline()
      while line: 
         print line

         line=infile.readline()
   
   def get_raw_target_len(self):
      ''' A function to get the length of the target sequence before it is processed by the alignment program. '''

      seq = ''
 
      file = open(self.input_fasta_file, 'r')

      line = file.readline()
      name=string.strip(line).replace('>','') 
      line = file.readline()
      while line:
         if '>' in line:
            break
         else:
           seq = seq + string.strip(line)
         line = file.readline()
    
      return len(seq)
         
  
if __name__ == '__main__':

   clustalw=Clustalw_multiple()
   clustalw.Clustalw('input/1vrd_all.fasta', 'out_1vrd.pir')
   clustalw.convert_to_fasta()



