#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#

import os, string, sys, time
import sets 
import Get_MR_sum


class Queue_info:
   """ A class to handle batch queue system interactions. """

   def __init__(self):

      self.queue_job_array=sets.Set([])
      self.job_completion=False
      self.finished_jobs=[]
      self.latest_jobs=[]
      self.RESULT=False
      self.marginal_solution = False
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag 

   def qstat_check(self, init, mstat, target_info, JOB_TYPE):
      """ A function to test the SGE for completion of a list of jobs denoted by
      their job ID's. Needs a "set" of job id's as input. """ 

      # Continuosly check the queue to see if our jobs have finished
      while self.job_completion == False:
         q=os.popen('qstat', 'r')
         header1=q.readline()
         header2=q.readline()
         line=q.readline() 

         log_file = open(mstat.search_dir + '/bulkMR.log','w')
	    
         while line:
            self.queue_job_array.add((int)((string.split(string.strip(line)))[0]))
            line=q.readline() 

         q.close()

         # Check the cluster queue to see if there are any jobs still running.
         # If there are jobs still running find out which ones have just completed.
         if mstat.jobid_array.intersection(self.queue_job_array):
             
            if JOB_TYPE == 'MOL_REPLACE' and mstat.solution_found == False:
               for job_name in self.latest_jobs:
                  self.finished_jobs.append(job_name)

               self.latest_jobs=[]
               for job in mstat.jobid_array.difference(self.queue_job_array):
                  if mstat.jobid_dict[job] not in self.finished_jobs:
                     self.latest_jobs.append(mstat.jobid_dict[job])
                
               self.check_result_parallel(init, mstat, target_info)
                
                
            log_file.write("%d jobs still running/pending at %s" % (len(mstat.jobid_array.intersection(self.queue_job_array)), time.ctime()))
            #print "%s jobs still running/pending" % len(mstat.jobid_array.intersection(self.queue_job_array))
            # print "ID's of jobs still running/pending: %s" % mstat.jobid_array.intersection(self.queue_job_array)
            time.sleep(10)
            self.queue_job_array.clear()
        
         # Otherwise, process the jobs that have just completed and finish. 
         else:
            self.job_completion=True

            if JOB_TYPE == 'MOL_REPLACE' and mstat.solution_found == False:
               for job_name in self.latest_jobs:
                  self.finished_jobs.append(job_name)

               self.latest_jobs=[]
               for job in mstat.jobid_array.difference(self.queue_job_array):
                  if mstat.jobid_dict[job] not in self.finished_jobs:
                     self.latest_jobs.append(mstat.jobid_dict[job])
                
               self.check_result_parallel(init, mstat, target_info)

            if mstat.solution_found == False and JOB_TYPE == 'MOL_REPLACE' and mstat.MRPROGRAM == 'MOLREP':
               print "No solution found using Molrep, trying Phaser."
            elif mstat.solution_found == False and JOB_TYPE == 'MOL_REPLACE' and mstat.MRPROGRAM == 'PHASER':
               print "No solution found using Phaser."
               print "All jobs complete"
            else:
               print "All jobs complete"

      # reset the job_completion flag and the finished and latest job arrays.
      self.job_completion=False
      self.finished_jobs=[]
      self.latest_jobs=[]
      log_file.close()


   def check_result_parallel(self, init, mstat, target_info):
      """ A function to check the refinement results for the finished jobs in parallel run."""

      # Loop over the list of models that have justed completed refinement.
      for job in self.latest_jobs:

         # Get the model name
         model=mstat.jobname_dict[job]

         sys.stdout.write("\n")
         sys.stdout.write("################################################################\n")
         sys.stdout.write("Job '%s' has completed for model '%s'\n" % (job, model))
         sys.stdout.write("################################################################\n")
         sys.stdout.write("\n")

         # If the score file exists then we can assume that the refinement completed succesfully.
         # Will change this to look at terminal signals later.
         if self.debug:
            print "Checking to see if Refmac produced a log file for model: " + model + "...."
            print "File name: " + mstat.model_list[model].refmac_logfile
            print ""

         if os.path.isfile(mstat.model_list[model].refmac_logfile):
            if self.debug:
               print "File exists"
               print ""
            log_file=open(mstat.model_list[model].refmac_logfile)

            # Set the model parameters associated with refinement
            mstat.model_list[model].setRefinement_Program('Refmac')
            score_found=self.getRefmacScores(model, mstat, mstat.model_list[model].refmac_logfile)
           
            if score_found:
               # Set some temporary variables to hold the refmac scores
               init_rfree  = mstat.model_list[model].refmac_init_score
               final_rfree = mstat.model_list[model].refmac_finl_score
               diff        = final_rfree/init_rfree

               # If the final Rfree is less than 0.3 then we can asssume that a solution has been found.
               if final_rfree <= 0.35:
                  print "Template Model: %s has produced a solution, final Rfree = %.3f" % (model, final_rfree)
                  print "removing all other jobs...."
                  for job in mstat.jobid_array:
                     os.system("qdel " + `job` + " > /dev/null") 
                  mstat.setSolutionFound(True)
                  mstat.setSolutionModel(model)

               # If the Rfree score falls by more than 20 % and the final value is less than .5 we can 
               # assume a successful refinement  
               elif final_rfree <= 0.5 and diff <= 0.8: 
                  print "Template Model: %s has produced a solution, final Rfree = %.3f and is %.3f of initial Rfree" % (model, final_rfree, diff)
                  print "removing all other jobs...."
                  for job in mstat.jobid_array:
                     os.system("qdel " + `job` + " > /dev/null") 
                  mstat.setSolutionFound(True)
                  mstat.setSolutionModel(model)
        
               elif final_rfree <= 0.5 and diff <=0.95:
                  print "Template Model: %s has produced a marginal solution, final Rfree = %.3f and is %.3f of initial Rfree" % (model, final_rfree, diff)
                  # Write the details to the results HTML 
                  mstat.mr_results.write_marginal_soln(init, mstat, model, final_rfree, diff) 
   
                  if mstat.MRPROGRAM == 'MOLREP':
                     print "Molrep log file:", mstat.model_list[model].molrep_logfile
                     print "Molrep PDB file:", mstat.model_list[model].molrep_PDBfile
                  elif mstat.MRPROGRAM == 'PHASER':
                     print "Phaser log file:", mstat.model_list[model].phaser_logfile
                     print "Phaser PDB file:", mstat.model_list[model].phaser_PDBfile
                  print "Refmac log file:", mstat.model_list[model].refmac_logfile
                  print "Refmac PDB file:", mstat.model_list[model].refmac_PDBfile
                  print ""
                  self.marginal_solution = True

               else:
                  print "Template Model: %s failed to produce a solution (Final Rfree = %.3f)" % (model, final_rfree)
                  if mstat.MRPROGRAM == 'MOLREP':
                     print "Molrep log file: %s" % mstat.model_list[model].molrep_logfile
                  elif mstat.MRPROGRAM == 'PHASER':
                     print "Phaser log file: %s" % mstat.model_list[model].phaser_logfile
                  print "Refmac log file: %s" % mstat.model_list[model].refmac_logfile
                  print ""
                  self.marginal_solution = False

               # If a solution has been found determine the Molecular Replacement details.
               if mstat.solution_found == True or self.marginal_solution == True:
                
                  # Molrep. Test to see if a PDB file was created using Molrep and set program variable.
	          if mstat.MRPROGRAM == "MOLREP":
                     if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
                        if self.debug:
                           print "Molrep PDB file exists"
                           print "PDB file name: " + mstat.model_list[model].molrep_PDBfile
                           print ""
                        mstat.model_list[model].setMR_Program('Molrep')
      
                        # Read and set the Molrep variables from the solution file.
                        if os.path.isfile(mstat.model_list[model].molrep_solnfile):
                           if self.debug:
                              print "Molrep solution file exists"
                              print "File name: " + mstat.model_list[model].molrep_solnfile
                              print ""
   
                           # Get the solution summary:
                           sum=Get_MR_sum.MR_Summary() 
                           sum.get_molrep_summary(mstat.model_list[model].molrep_solnfile)
                           mstat.model_list[model].setMolrepSummary(sum.molrep_summary)               
                           print ""
                           print "#####################################################################"
                           print "#####           Summary of top solutions from Molrep:           #####"
                           print "#####################################################################"
                           print ""
                           print mstat.model_list[model].molrep_summary
   
                           log=open(mstat.model_list[model].molrep_solnfile)
                           line=log.readline()
   
                           while line:
                              if 'S_ Nmon RF TF   theta    phi    chi     tx     ty     tz    TFcnt   Rfac' in line:
                                 line=log.readline()
                                 mstat.model_list[model].setMolrepCorr(float(string.split(line)[-1]))
                                 mstat.model_list[model].setMolrepRfac(float(string.split(line)[-2]))
                                 break
   
                              line=log.readline()
   
                           log.close()
   
                        else:
                           if self.debug:
                              print "Molrep solution file not found for:", model
                              print "File name: " + mstat.model_list[model].molrep_solnfile
                              print ""

                     else:
                        if self.debug:
                           print "Molrep PDB file not found for:", model
                           print "PDB File name: " + mstat.model_list[model].molrep_PDBfile
                           print ""

                  # Likewise for Phaser. Phaser will superseed Molrep if it has been necessary to use it.
                  if mstat.MRPROGRAM == "PHASER":
                     if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
                        if self.debug:
                           print "Phaser PDB file exists"
                           print "PDB file name: " + mstat.model_list[model].phaser_PDBfile
                           print ""
                        mstat.model_list[model].setMR_Program('Phaser')
   
                        # Read and set the Molrep variables from the solution file.
                        if os.path.isfile(mstat.model_list[model].phaser_solnfile):
                           if self.debug:
                              print "Phaser solution file exists"
                              print "File name: " + mstat.model_list[model].phaser_solnfile
                              print ""
   
                           # Get the solution summary:
                           sum=Get_MR_sum.MR_Summary() 
                           sum.get_phaser_summary(mstat.model_list[model].phaser_summary)
                           mstat.model_list[model].setPhaserSummary(sum.phaser_summary)               
      
                           sys.stdout.write("\n")
                           sys.stdout.write("#####################################################################\n")
                           sys.stdout.write("#####     Results from final translation function in Phaser:    #####\n")
                           sys.stdout.write("#####################################################################\n")
                           sys.stdout.write("\n")
                           print mstat.model_list[model].phaser_summary
   
                           log=open(mstat.model_list[model].phaser_solnfile)
                           line=log.readline()
   
                           while line:
                              if 'LLG' in line:
                                 mstat.model_list[model].setPhaserLLGscore(float(string.split(line)[-1].replace('LLG=','')))
                                 # Put in Zscore aswell here. 
                                 break
   
                              line=log.readline()
   
                           log.close()
   
                        else:
                           if self.debug:
                              print "Phaser solution file not found for:", model
                              print "File name: " + mstat.model_list[model].phaser_solnfile
                              print ""
   
                     else:
                        if self.debug:
                           print "Phaser PDB file not found for:", model
                           print "PDB File name: " + mstat.model_list[model].phaser_PDBfile
                           print ""

                  if os.path.isfile(mstat.model_list[model].refmac_logfile):
                     # Output the refinement scores from the refmac log file 
                     sum.get_refmac_summary(mstat.model_list[model].refmac_logfile)
                     mstat.model_list[model].setRefmacSummary(sum.refmac_summary)               
                     print ""
                     print "#####################################################################"
                     print "#####              Refinement results from Refmac:              #####"
                     print "#####################################################################"
                     print ""
                     print mstat.model_list[model].refmac_summary
                     print ""
                  else:
                     if self.debug:
                        print "Refmac logfile not found for:", model

                  # If we now have a solution we can break out of the model list loop.
                  if mstat.solution_found:
                     mstat.mr_results.copy_solution_files(mstat, mstat.results_dir + '/solution')
                     mstat.mr_results.display_results(mstat, target_info)
                     break
	          else:
                     sys.stdout.write("*******************************************************************************************\n")
                     sys.stdout.write("\n")


         # If no score file exists then we can assume that Refinement/MR failed and continue to the next 
         # model.
         else:
            print "Refmac log file not found for job number: %d, model ID: %s" % (mstat.model_list[model].MR_job_number, model)
            if mstat.MRPROGRAM == 'MOLREP':
               print "Molrep log file: %s" % mstat.model_list[model].molrep_logfile
            elif mstat.MRPROGRAM == 'PHASER':
               print "Phaser log file: %s" % mstat.model_list[model].phaser_logfile
            print "Refmac log file: %s" % mstat.model_list[model].refmac_logfile
            print "\n"


   def check_result_serial(self, mstat, model):
      """ A function to check the refinement results for the finished jobs in a serial run."""

      # If the refmac log file exists then we can assume that the refinement completed succesfully.
      # Will change this to look at terminal signals later.
      sys.stdout.write("\n")
      sys.stdout.write("################################################################\n")
      sys.stdout.write("Job has completed for model '%s'\n" %  model)
      sys.stdout.write("################################################################\n")
      sys.stdout.write("\n")

      if self.debug:
         print "Checking to see if Refmac produced a log file for model: " + model + "...."
         print "File name: " + mstat.model_list[model].refmac_logfile
         print ""
  
      if os.path.isfile(mstat.model_list[model].refmac_logfile):
         if self.debug:
            print "File exists"
            print ""
         log_file=open(mstat.model_list[model].refmac_logfile)

         # Set the model parameters associated with refinement
         mstat.model_list[model].setRefinement_Program('Refmac')
         score_found=self.getRefmacScores(model, mstat, mstat.model_list[model].refmac_logfile)

         if score_found:
            # Set some temporary variables to hold the refmac scores
            init_rfree  = mstat.model_list[model].refmac_init_score
            final_rfree = mstat.model_list[model].refmac_finl_score
            diff        = final_rfree/init_rfree
   
            # If the final Rfree is less than 0.3 then we can asssume that a solution has been found.
            if final_rfree <= 0.35:
               print "Template Model: %s has produced a solution, final Rfree = %.3f" % (model, final_rfree)
               print ""
               mstat.setSolutionFound(True)
               mstat.setSolutionModel(model)
   
            # If the Rfree score falls by more than 20 % and the final value is less than .5 we can 
            # assume a successful refinement  
            elif final_rfree <= 0.5 and diff <= 0.8: 
               print "Template Model: %s has produced a solution, final Rfree = %.3f and is %.3f of initial Rfree" % (model, final_rfree, diff)
               print ""
               mstat.setSolutionFound(True)
               mstat.setSolutionModel(model)
   
            elif final_rfree <= 0.5 and diff <=0.95:
               print "Template Model: %s has produced a marginal solution, final Rfree = %.3f and is %.3f of initial Rfree" % (model, final_rfree, diff)
               print ""
               if mstat.MRPROGRAM == 'MOLREP':
                  print "Molrep log file:", mstat.model_list[model].molrep_logfile
                  print "Molrep PDB file:", mstat.model_list[model].molrep_PDBfile
               elif mstat.MRPROGRAM == 'PHASER':
                  print "Phaser log file:", mstat.model_list[model].phaser_logfile
                  print "Phaser PDB file:", mstat.model_list[model].phaser_PDBfile
               print "Refmac log file:", mstat.model_list[model].refmac_logfile
               print "Refmac PDB file:", mstat.model_list[model].refmac_PDBfile
               print ""
               self.marginal_solution = True
   
            else:
               print "Template Model: %s failed to produce a solution (Final Rfree = %.3f)" % (model, final_rfree)
               print ""
               if mstat.MRPROGRAM == 'MOLREP':
                  print "Molrep log file: %s" % mstat.model_list[model].molrep_logfile
               elif mstat.MRPROGRAM == 'PHASER':
                  print "Phaser log file: %s" % mstat.model_list[model].phaser_logfile
               print "Refmac log file: %s" % mstat.model_list[model].refmac_logfile
               print ""
               self.marginal_solution = False 
   
            # If a solution has been found determine the Molecular Replacement details.
            if mstat.solution_found == True or self.marginal_solution == True:
                   
               # Molrep. Test to see if a PDB file was created using Molrep and set program variable.
               if mstat.MRPROGRAM =="MOLREP":
   
                  if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
                     if self.debug:
                        print "Molrep PDB file exists"
                        print "PDB file name: " + mstat.model_list[model].molrep_PDBfile
                        print ""
                     mstat.model_list[model].setMR_Program('Molrep')
   
                     # Read and set the Molrep variables from the solution file.
                     if os.path.isfile(mstat.model_list[model].molrep_solnfile):
                        if self.debug:
                           print "Molrep solution file exists"
                           print "File name: " + mstat.model_list[model].molrep_solnfile
                           print ""
   
                        # Get the solution summary:
                        sum=Get_MR_sum.MR_Summary() 
                        sum.get_molrep_summary(mstat.model_list[model].molrep_solnfile)
                        mstat.model_list[model].setMolrepSummary(sum.molrep_summary)               
                        print ""
                        print "#####################################################################"
                        print "#####           Summary of top solutions from Molrep:           #####"
                        print "#####################################################################"
                        print ""
                        print mstat.model_list[model].molrep_summary
   
                        # Open the log file and extract the scores (move this to a seperate class at some stage)
                        log=open(mstat.model_list[model].molrep_solnfile)
                        line=log.readline()
   
                        while line:
                           if 'S_ Nmon RF TF   theta    phi    chi     tx     ty     tz    TFcnt   Rfac' in line:
                              line=log.readline()
                              mstat.model_list[model].setMolrepCorr(float(string.split(line)[-1]))
                              mstat.model_list[model].setMolrepRfac(float(string.split(line)[-2]))
                              break

                           line=log.readline()
   
                        log.close()
   
                     else:
                        if self.debug:
                           print "Molrep solution file not found for:", model
                           print "File name: " + mstat.model_list[model].molrep_solnfile
                           print ""
   
                  else:
                     if self.debug:
                        print "Molrep PDB file not found for:", model
                        print "PDB File name: " + mstat.model_list[model].molrep_PDBfile
                        print ""
   
                # Likewise for Phaser. Phaser will superseed Molrep if it has been necessary to use it.
               if mstat.MRPROGRAM =="PHASER":
   
                  if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
                     if self.debug:
                        print "Phaser PDB file exists"
                        print "PDB file name: " + mstat.model_list[model].phaser_PDBfile
                        print ""
                     mstat.model_list[model].setMR_Program('Phaser')
   
                     # Read and set the Molrep variables from the solution file.
   
                     if os.path.isfile(mstat.model_list[model].phaser_solnfile):
                        if self.debug:
                           print "Phaser solution file exists"
                           print "File name: " + mstat.model_list[model].phaser_solnfile
                           print ""
   
                        # Get the solution summary:
                        sum=Get_MR_sum.MR_Summary() 
                        sum.get_phaser_summary(mstat.model_list[model].phaser_summary)
                        mstat.model_list[model].setPhaserSummary(sum.phaser_summary)               
                        print ""
                        print "#####################################################################"
                        print "#####     Results from final translation function in Phaser:    #####"
                        print "#####################################################################"
                        print ""
                        print mstat.model_list[model].phaser_summary
   
                        # Open the log file and extract the scores (move this to a seperate class at some stage)
                        log=open(mstat.model_list[model].phaser_solnfile)
                        line=log.readline()
   
                        while line:
                           if 'LLG' in line:
                              mstat.model_list[model].setPhaserLLGscore(float(string.split(line)[-1].replace('LLG=','')))
                              # Put in Zscore aswell here. 
                              break
   
                           line=log.readline()

                        log.close()
   
                     else:
                        if self.debug:
                           print "Phaser solution file not found for:", model
                           print "File name: " + mstat.model_list[model].phaser_solnfile
                           print ""
   
                  else:
                     if self.debug:
                        print "Phaser PDB file not found for:", model
                        print "PDB File name: " + mstat.model_list[model].phaser_PDBfile
                        print ""
   
               if os.path.isfile(mstat.model_list[model].refmac_logfile):
                  # Output the refinement scores from the refmac log file 
                  sum.get_refmac_summary(mstat.model_list[model].refmac_logfile)
                  mstat.model_list[model].setRefmacSummary(sum.refmac_summary)               
                  print ""
                  print "#####################################################################"
                  print "#####              Refinement results from Refmac:              #####"
                  print "#####################################################################"
                  print ""
                  print mstat.model_list[model].refmac_summary
                  print ""
               else:
                  if self.debug:
                     print "Refmac logfile not found for:", model

      # If no score file exists then we can assume that Refinement/MR failed and continue to the next 
      # model.
      else:
         print "Refmac log file not found for:", model
         print ""
         print "Checking MR output..."

         if mstat.MRPROGRAM == "MOLREP":
            if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
               print "Molrep produced a PDB, error was in Refmac"
               print "Check the refmac logfile:"
               print mstat.model_list[model].refmac_logfile
            else:
               print "Molrep failed to produce a PDB"
               print "Check the molrep logfile:"
               print mstat.model_list[model].molrep_logfile

         if mstat.MRPROGRAM == "PHASER":
            if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
               print "Phaser produced a PDB, error was in Refmac"
               print "Check the refmac logfile:"
               print mstat.model_list[model].refmac_logfile
            else:
               print "Phaser failed to produce a PDB"
               print "Check the phaser logfile:"
               print mstat.model_list[model].phaser_logfile

         
   def getRefmacScores(self, model, mstat, refmac_logfile):
      """ Extract the relevant scores from the refmac log file. """

      score_found = False
      log=open(refmac_logfile,'r')

      # Loop over the file and extract the initial and final Rfree values
      line=log.readline()
      while line:
         if 'Ncyc' in line and 'Rfact' in line:

            line=log.readline()
            line=log.readline()

            mstat.model_list[model].setRefmacInitialScore(float(string.split(line)[2]))

            while '$$' not in line:
               previous_line=line
               line=log.readline()
            mstat.model_list[model].setRefmacFinalScore(float(string.split(previous_line)[2]))
          
            score_found = True

         line=log.readline()
      log.close()
      
      # Check that the scores were found
      if score_found == False:
         sys.stdout.write("Refmac Scoring Error: Initial and Final Rfree values were not found for model: " + model + "\n") 
         sys.stdout.write("Refmac Scoring Error: Logfile: \n") 
         sys.stdout.write("                " + refmac_logfile + "\n") 
         sys.stdout.write("\n") 

      return score_found
